#! /usr/bin/perl -wT
################################################################################
#
# CGI Helper Class
#
# File   :  CGI.pm
#
################################################################################
#
#    This file includes parts which are copyrighted by Adaptive Computing
#    Enterprises, Inc., and parts which are copyrighted by Battelle Memorial
#    Institute.
#
#    The terms and conditions for the use and redistribution of these parts
#    are governed by the Moab Accounting Manager License and the BSD License
#    respectively. Refer to the LICENSE file for details.
#
#    Copyright (C) 2006 - 2016 by Adaptive Computing Enterprises, Inc.
#    All Rights Reserved.
#
#    The Moab Accounting Manager License specifies the terms and conditions
#    for use and redistribution.
#
#    The Moab Accounting Manager License applies to the Moab Accounting
#    Manager software offered by Adaptive Computing Enterprises, Inc.
#    By installing or using this software, Licensee accepts a non-exclusive
#    license from Adaptive Computing Enterprises, Inc. and is bound to accept
#    acknowledgement of and abide by the notices and conditions of the
#    Moab Accounting Manager License.
#
#    Copyright (C) 2003 - 2005 Pacific Northwest National Laboratory,
#    Battelle Memorial Institute. All rights reserved.
#
#    The BSD License specifies the terms and conditions for use and
#    redistribution.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#
#    - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
#    - Neither the name of Battelle nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#    POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

use strict;

=head1 NAME

MAM::CGI - A Helper class for MAM's CGI web-based Graphical User Interface

=head1 DESCRIPTION

The B<MAM::CGI> module is a helper class to provide the CGI environment with distinct log and configuration files.

=head1 EXAMPLES

use MAM::CGI;

=head1 REQUIRES

Perl 5.8

=head1 AUTHOR

Scott Jackson, scottmo@adaptivecomputing.com

=cut

##############################################################################
package MAM::CGI;

use vars
  qw($clientConfig $format $localSiteConfig $log $quiet $time_division $verbose $VERSION &toHRT &toMFT);
use Carp;
use Exporter;
@MAM::CGI::ISA = qw(Exporter);
@MAM::CGI::EXPORT =
  qw($clientConfig $localSiteConfig $format $log $quiet $time_division $verbose $VERSION &toHRT &toMFT);

use lib "/opt/mam/lib";
use lib "/opt/mam/lib/perl5";
use Log::Log4perl qw(get_logger :levels);
use Config::Tiny;
use MAM::Global;
use MAM::Assignment;
use MAM::Condition;
use MAM::Selection;
use MAM::Option;
use MAM::Datum;
use MAM::Request;
use MAM::Response;
use MAM::Message;
use MAM::Reply;
use MAM::Client;

BEGIN
{
    # Read the Config File
    my $clientConfigFile = "/opt/mam/etc/mam-gui.conf";
    my $clientConfigTiny = Config::Tiny->new;
    $clientConfigTiny = Config::Tiny->read($clientConfigFile)
      or die(
        "Unable to open the gui configuration file ($clientConfigFile) for reading: $Config::Tiny::errstr\n");
    $clientConfig = $clientConfigTiny->{_};

    # Configure Logging
    Log::Log4perl::Logger::create_custom_level('TRACE', 'DEBUG')
      unless Log::Log4perl::Level::is_valid('TRACE');
    Log::Log4perl->init("$clientConfigFile");
    $log = get_logger('MAM');
    $log->info(
        "$0 (PID $$) Started *******************************************");

    # Read the Site Config File if suidperl promotion is being used
    my $promotionMethod = $clientConfig->{'security.promotion'}
      || $SECURITY_PROMOTION;
    if ($promotionMethod eq 'suidperl')
    {
        my $siteConfigFile = "/opt/mam/etc/mam-site.conf";
        my $siteConfigTiny = Config::Tiny->new;
        $siteConfigTiny = Config::Tiny->read($siteConfigFile)
          or die(
            "Unable to open the site configuration file ($siteConfigFile) for reading: $!\n"
          );
        $localSiteConfig = $siteConfigTiny->{_};
    }
}

sub untaint
{
    my ($username_ref, $password_ref) = @_;
    my @messages = ();

    if ($$username_ref =~ /^([-\w]+)$/)
    {
        $$username_ref = $1;
    }    # Untaint $username
    else
    {
        push @messages,
          "Illegal characters were found in the user name ($$username_ref).";
    }

    if ($$password_ref =~ /^(.*)$/) { $$password_ref = $1; } # Untaint $password
    else { push @messages, 'Illegal characters were found in the password.'; }

    return join '\n', @messages;
}

1;
