#! /usr/bin/perl -wT
################################################################################
#
# Global constants
#
# File   :  Global.pm
#
################################################################################
#
#    This file includes parts which are copyrighted by Adaptive Computing
#    Enterprises, Inc., and parts which are copyrighted by Battelle Memorial
#    Institute.
#
#    The terms and conditions for the use and redistribution of these parts
#    are governed by the Moab Accounting Manager License and the BSD License
#    respectively. Refer to the LICENSE file for details.
#
#    Copyright (C) 2006 - 2016 by Adaptive Computing Enterprises, Inc.
#    All Rights Reserved.
#
#    The Moab Accounting Manager License specifies the terms and conditions
#    for use and redistribution.
#
#    The Moab Accounting Manager License applies to the Moab Accounting
#    Manager software offered by Adaptive Computing Enterprises, Inc.
#    By installing or using this software, Licensee accepts a non-exclusive
#    license from Adaptive Computing Enterprises, Inc. and is bound to accept
#    acknowledgement of and abide by the notices and conditions of the
#    Moab Accounting Manager License.
#
#    Copyright (C) 2003 - 2005 Pacific Northwest National Laboratory,
#    Battelle Memorial Institute. All rights reserved.
#
#    The BSD License specifies the terms and conditions for use and
#    redistribution.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#
#    - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
#    - Neither the name of Battelle nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#    POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

use strict;

=head1 NAME

MAM::Global - defines global constants for MAM

=head1 DESCRIPTION

The B<MAM::Global> module defines constants used as default values for config parameters

=head1 EXAMPLES

use MAM::Global;

print "$SERVER_PORT\n";

=head1 REQUIRES

Perl 5.8

=head1 AUTHOR

Scott Jackson, scottmo@adaptivecomputing.com

=cut

##############################################################################

package MAM::Global;

use Exporter;
use Time::Local;
use Time::HiRes qw(gettimeofday);
@MAM::Global::ISA    = qw(Exporter);
@MAM::Global::EXPORT = qw(
  $clientConfig
  $globalSiteConfig
  $localSiteConfig
  $log
  $quiet
  $format
  $requestTime
  $serverConfig
  $time_division
  $verbose
  $ACCOUNTING_MODE
  $DB_DATASOURCE
  $EVENT_SCHEDULER
  $POLL_INTERVAL
  $PRODUCT
  $RESPONSE_CHUNKING
  $RESPONSE_CHUNKSIZE
  $SBIN_DIR
  $SECURITY_AUTHENTICATION
  $SECURITY_ENCRYPTION
  $SECURITY_PROMOTION
  $SERVER_HOST
  $SERVER_PORT
  $SHA_ALGORITHM
  $SUPER_USER
  $TOKEN_NONE
  $TOKEN_PASSWORD
  $TOKEN_SYMMETRIC
  $VERSION
  $WIRE_PROTOCOL
  &about
  &difference
  &intersection
  &lower
  &logAndDie
  &logAndWarn
  &max
  &min
  &toDBC
  &toDMF
  &toUCC
  &toHRT
  &toMFT
);
use vars qw(
  $clientConfig
  $globalSiteConfig
  $localSiteConfig
  $log
  $quiet
  $format
  $requestTime
  $serverConfig
  $time_division
  $verbose
  $ACCOUNTING_MODE
  $DB_DATASOURCE
  $EVENT_SCHEDULER
  $POLL_INTERVAL
  $PRODUCT
  $RESPONSE_CHUNKING
  $RESPONSE_CHUNKSIZE
  $SBIN_DIR
  $SECURITY_AUTHENTICATION
  $SECURITY_ENCRYPTION
  $SECURITY_PROMOTION
  $SERVER_HOST
  $SERVER_PORT
  $SHA_ALGORITHM
  $SUPER_USER
  $TOKEN_NONE
  $TOKEN_PASSWORD
  $TOKEN_SYMMETRIC
  $VERSION
  $WIRE_PROTOCOL
  &about
  &difference
  &intersection
  &logAndDie
  &logAndWarn
  &lower
  &max
  &min
  &toDBC
  &toDMF
  &toUCC
  &toHRT
  &toMFT
);

# ${*}Config are globals for manipulating config file parameters
# $quiet, $format, $time_division and $verbose are global variables for output formatting

# Format defaults to standard unless overridden with --format flag
$format = 'standard';

# Time division defaults to 1 unless overridden with --hours flag
$time_division = 1;

# Accounting Mode
$ACCOUNTING_MODE = 'strict-allocation';

# Database Type
my $DB_TYPE = 'Pg';

# Database Datasource
my $DB_DATASOURCE = 'DBI:Pg:dbname=mam;host=localhost';

# Enable event scheduler
$EVENT_SCHEDULER = 'true';

# Event poll interval
$POLL_INTERVAL = 5;

# Whether chunking should be performed
$RESPONSE_CHUNKING = 'false';

# Response Chunk Size
$RESPONSE_CHUNKSIZE = '1000';

# Installed sbin Directory
$SBIN_DIR = '/opt/mam/sbin';

# Whether authentication is performed by default
$SECURITY_AUTHENTICATION = 'true';

# Whether encryption is performed by default
$SECURITY_ENCRYPTION = 'false';

# Method used to promote privileges to read site.conf (mamauth or suidperl)
$SECURITY_PROMOTION = 'mamauth';

# Server Host
$SERVER_HOST = 'localhost';

# Server Port
$SERVER_PORT = '7112';

# SHA Algorithm
$SHA_ALGORITHM = '1';

# Super User
$SUPER_USER = 'mam';

# Asymmetric Security Token Type
my $TOKEN_ASYMMETRIC = 'Asymmetric';

# ClearText Password Security Token Type
my $TOKEN_CLEARTEXT = 'Cleartext';

# GSI Security Token Type
my $TOKEN_GSI = 'X509v3';

# Kerberos Security Token Type
my $TOKEN_KERBEROS = 'Kerberos5';

# No Security Token Type
$TOKEN_NONE = 'None';

# Known Password Security Token Type
$TOKEN_PASSWORD = 'Password';

# Symmetric Security Token Type
$TOKEN_SYMMETRIC = 'Symmetric';

# Product Name
$PRODUCT = 'Moab Accounting Manager';

# Product Version
$VERSION = '9.1.0';

# Wire Protocol [SSSRMAP, SOAP]
$WIRE_PROTOCOL = 'SSSRMAP';

# ----------------------------------------------------------------------------
# $dbname = toDBC($name)
# ----------------------------------------------------------------------------

# To Data Base Case
# Converts from the form CompoundName to g_compound_name
sub toDBC
{
    my ($name) = @_;

    $name =~ s/\b([A-Z])/g_\L$1/g;    # Replace all {word boundary}A with g_a
    $name =~ s/([A-Z])/_\L$1/g;       # Replace all A with _a

    return $name;
}

# ----------------------------------------------------------------------------
# $name = toUCC($dbname)
# ----------------------------------------------------------------------------

# To Upper Camel Case
# Converts from the form g_compound_name to CompoundName
sub toUCC
{
    my ($name) = @_;

    $name =~ s/^g(?=_)//;          # Remove leading g (if followed by _)
    $name =~ s/_([a-z])/\U$1/g;    # Replace all _a with A
    return $name;
}

# ----------------------------------------------------------------------------
# $dateTime = toHRT($epoch)
# ----------------------------------------------------------------------------

# To Human Readable Time
# Converts from epoch time to Human readable time format
sub toHRT
{
    my ($epoch) = @_;
    my $dateTime;

    if ($epoch =~ /^\d+$/)
    {
        if ($epoch == 0)
        {
            $dateTime = '-Infinity';
        }
        elsif ($epoch == 2147483647)
        {
            $dateTime = 'Infinity';
        }
        else
        {
            my ($sec, $min, $hour, $day, $month, $year) =
              (localtime($epoch))[0 .. 5];
            $dateTime =
              sprintf('%04d-%02d-%02d', $year + 1900, $month + 1, $day);
            if ($hour || $min || $sec)
            {
                $dateTime .= sprintf(' %02d:%02d:%02d', $hour, $min, $sec);
            }
        }
    }
    else
    {
        $dateTime = $epoch;
    }
    return $dateTime;
}

# ----------------------------------------------------------------------------
# $epoch = toMFT($dateTime)
# ----------------------------------------------------------------------------

# To Message Format Time
# Converts from human readable time to epoch time
sub toMFT
{
    my ($dateTime) = @_;
    my ($epoch);

    if ($dateTime =~ /-inf/i)
    {
        $epoch = 0;
    }
    elsif ($dateTime =~ /inf/i)
    {
        $epoch = 2147483647;
    }
    elsif ($dateTime =~ /now/i)
    {
        $epoch = ${[gettimeofday()]}[0];
    }
    elsif ($dateTime =~
        /(\d{4})-(\d{1,2})-(\d{1,2})( (\d{1,2}):(\d{1,2}):(\d{1,2}))?/)
    {
        my ($sec, $min, $hour, $day, $month, $year);
        ($year, $month, $day) = ($1, $2, $3);
        if ($4) { ($hour, $min, $sec) = ($5, $6, $7) }
        else    { ($hour, $min, $sec) = (0, 0, 0) }
        $epoch = timelocal($sec, $min, $hour, $day, $month - 1, $year - 1900);
    }
    else
    {
        $epoch = $dateTime;
    }
    return $epoch;
}

# ----------------------------------------------------------------------------
# $string = toDMF($rearmPeriod [, $baseTimeRef [,$deltaRef]])
# ----------------------------------------------------------------------------

# To Date::Manip Frequency
# Converts from MAM rearm period syntax to Date::Manip's ParseRecur
# frequency description string
# If you pass in references to a basetime and a delta, this routine
# will populate them with values for use in ParseRecur
sub toDMF
{
    my ($rearmPeriod, $baseTimeRef, $deltaRef) = @_;
    my ($year, $month, $week, $day, $hour, $min, $sec) = ('0', (':0') x 6);

    my ($period, $instant, $this, $sync, $now);
    (my $rearmString = $rearmPeriod) =~ s/\s//g;    # Remove all white space
    if ($rearmString =~ /^(\w+)(?:(?:(?:@(\w+))?(?:(~)|(\^))?)|(!))$/)
    {
        ($period, $instant, $this, $sync, $now) = ($1, $2, $3, $4, $5);
    }
    else
    {
        return
          "ReArm period ($rearmPeriod) must be of the form: <period>[[@<instant>][~|^]|!]";
    }

    # Parse modifier into origin
    # An origin of 'now' means relative to this minute
    # An origin of 'this' means relative to when this designator last changed
    # An origin of 'sync' means relative to when this designator became zero
    # 'this' and 'sync' will be the same if the period designator's value is 1
    # If, for example, the period is 4 hours and the time is 3:45,
    # 'now' will trigger next at 7:45, 'this' will trigger next at 7:00, and
    # 'sync' will trigger next at 4:00.
    my $origin = 'this';    # Default
    if    ($now)  { $origin = 'now'; }
    elsif ($sync) { $origin = 'sync'; }
    elsif ($this) { $origin = 'this'; }

    # Initialize frequency components
    my (%separators, %values, %prefixes, %suffixes);
    @separators{(1 .. 7)} = ((':') x 6, '');
    @values{(1 .. 7)}     = (0) x 7;
    @prefixes{(1 .. 7)}   = ();
    @suffixes{(1 .. 7)}   = ();
    my $delta = 0;

    # Set prefix values
    while ($period =~ s/^(\d+)([a-zA-Z]+)//)
    {
        my ($number, $designator) = ($1, $2);
        if ($designator =~ /^ye?a?r?s?$/i)
        {
            $prefixes{7} = $number;
            $delta += $number * 366 * 86400;
        }
        elsif ($designator =~ /^mon?t?h?s?$/i)
        {
            $prefixes{6} = $number;
            $delta += $number * 31 * 86400;
        }
        elsif ($designator =~ /^we?e?k?s?$/i)
        {
            $prefixes{5} = $number;
            $delta += $number * 7 * 86400;
        }
        elsif ($designator =~ /^da?y?s?$/i)
        {
            $prefixes{4} = $number;
            $delta += $number * 86400;
        }
        elsif ($designator =~ /^ho?u?r?s?$/i)
        {
            $prefixes{3} = $number;
            $delta += $number * 3600;
        }
        elsif ($designator =~ /^min?u?t?e?s?$/i)
        {
            $prefixes{2} = $number;
            $delta += $number * 60;
        }
        else { return "Invalid period designator ($designator)"; }
    }
    return "Invalid period expression ($period)" unless $period eq '';
    foreach my $place (keys %prefixes)
    {
        $values{$place} = $prefixes{$place} if defined $prefixes{$place};
    }
    my $minPrefix = min(grep { $prefixes{$_} } (keys %prefixes));
    my $maxPrefix = max(grep { $prefixes{$_} } (keys %prefixes));
    $$deltaRef = $delta if defined $deltaRef;

    # Synchronized rearm periods
    if ($origin eq 'sync' || $origin eq 'this')
    {
        # Set the period/instant marker
        $separators{$minPrefix - 1} = '*';

        # Preset month and day to 1
        foreach my $place (4, 6)
        {
            if ($minPrefix > $place) { $values{$place} = 1; }
        }

        # Set the instant
        if (defined $instant)
        {
            # Set suffix values
            while ($instant =~ s/^([a-zA-Z]+)(\d+)//)
            {
                my ($designator, $number) = ($1, $2);
                if ($designator =~ /^mon?t?h?s?$/i) { $suffixes{6} = $number; }
                elsif ($designator =~ /^we?e?k?s?$/i)
                {
                    $suffixes{5} = $number;
                }
                elsif ($designator =~ /^da?y?s?$/i) { $suffixes{4} = $number; }
                elsif ($designator =~ /^ho?u?r?s?$/i)
                {
                    $suffixes{3} = $number;
                }
                elsif ($designator =~ /^min?u?t?e?s?$/i)
                {
                    $suffixes{2} = $number;
                }
                else { return "Invalid instant designator ($designator)"; }
            }
            return "Invalid instant expression ($instant)"
              unless $instant eq '';
            my $maxSuffix = max(grep { $suffixes{$_} } (keys %suffixes));
            return
              'The largest instant designator must be smaller than the smallest period designator'
              unless $maxSuffix < $minPrefix;
            foreach my $place (keys %suffixes)
            {
                $values{$place} = $suffixes{$place}
                  if defined $suffixes{$place};
            }
        }
    }
    else
    {
        # Zero out seconds only
        $separators{1} = '*';
    }

    # Adjust baseTime ref if origin is sync with a plural period designator
    if (defined $baseTimeRef && $origin eq 'sync' && $prefixes{$maxPrefix} > 1)
    {
        my ($sec, $min, $hour, $day, $month, $year) =
          (localtime($$baseTimeRef))[0 .. 5];
        if    ($maxPrefix == 7) { $year  = 100; }
        elsif ($maxPrefix == 6) { $month = 0; }
        elsif ($maxPrefix == 5) { $month = 0; }
        elsif ($maxPrefix == 4) { $day   = 0; }
        elsif ($maxPrefix == 3) { $hour  = 0; }
        elsif ($maxPrefix == 2) { $min   = 0; }
        $$baseTimeRef = timelocal($sec, $min, $hour, $day, $month, $year);
    }

    # Build and return frequency description string
    my $string = '';
    foreach my $place (reverse(1 .. 7))
    {
        $string .= $separators{$place} . $values{$place};
    }
    return $string;
}

# ----------------------------------------------------------------------------
# $lowball = min(@numbers)
# ----------------------------------------------------------------------------

# Return minimum from a list of numbers
sub min
{
    my (@numbers) = @_;

    my ($min);

    $min = $numbers[0];
    foreach my $i (@numbers)
    {
        if ($i < $min)
        {
            $min = $i;
        }
    }

    return $min;
}

# ----------------------------------------------------------------------------
# $highball = max(@numbers)
# ----------------------------------------------------------------------------

# Return maximum from a list of numbers
sub max
{
    my (@numbers) = @_;

    my ($max);

    $max = $numbers[0];
    foreach my $i (@numbers)
    {
        if ($i > $max)
        {
            $max = $i;
        }
    }

    return $max;
}

# ----------------------------------------------------------------------------
# $lowerSpacedString = lower($camelCaseString)
# ----------------------------------------------------------------------------

# Transform an upper camel case string to a lower case space-separated string
sub lower
{
    my ($ccs) = @_;

    my $lcs = lcfirst $ccs;
    $lcs =~ s/([A-Z])/ \L$1\E/g;

    return $lcs;
}

# ----------------------------------------------------------------------------
# @difference = difference(\@a, \@b)
# ----------------------------------------------------------------------------

# Return the difference of two arrays (A - B)
# That is take
sub difference
{
    my ($a_ref, $b_ref) = @_;

    my %seenInB = map { $_ => 1 } @{$b_ref};

    return grep {! $seenInB{$_} } @{$a_ref};
}

# ----------------------------------------------------------------------------
# @intersection = intersection(\@a, \@b)
# ----------------------------------------------------------------------------

# Return the intersection of two arrays
# Keeps the order of the second parameter
sub intersection
{
    my ($a_ref, $b_ref) = @_;

    my %seenInA = map { $_ => 1 } @{$a_ref};

    return grep { $seenInA{$_} } @{$b_ref};
}

# ----------------------------------------------------------------------------
# &logAndDie( $message )
# ----------------------------------------------------------------------------

sub logAndDie
{
    my ($message) = @_;
    $log->fatal($message);
    die("$message\n");
}

# ----------------------------------------------------------------------------
# &logAndWarn( $message )
# ----------------------------------------------------------------------------

sub logAndWarn
{
    my ($message) = @_;
    $log->error($message);
    print STDERR "$message\n";
}

# ----------------------------------------------------------------------------
# $aboutString = about()
# ----------------------------------------------------------------------------

# Return build information
sub about
{
    return <<"EOF";
Product Name:       $PRODUCT
Product Version:    $VERSION
Revision Number:    bf2c35d491bac4a665b8e8b832387de0d712f70e
Install Directory:  /opt/mam
Home Directory:     /opt/mam
EOF
}

1;
