#! /usr/bin/perl -w
################################################################################
#
# Authentication Handler for MAM Web Services
#
# File   :  WSAuthenHandler.pm
#
################################################################################
#
#    This file includes parts which are copyrighted by Adaptive Computing
#    Enterprises, Inc., and parts which are copyrighted by Battelle Memorial
#    Institute.
#
#    The terms and conditions for the use and redistribution of these parts
#    are governed by the Moab Accounting Manager License and the BSD License
#    respectively. Refer to the LICENSE file for details.
#
#    Copyright (C) 2006 - 2016 by Adaptive Computing Enterprises, Inc.
#    All Rights Reserved.
#
#    The Moab Accounting Manager License specifies the terms and conditions
#    for use and redistribution.
#
#    The Moab Accounting Manager License applies to the Moab Accounting
#    Manager software offered by Adaptive Computing Enterprises, Inc.
#    By installing or using this software, Licensee accepts a non-exclusive
#    license from Adaptive Computing Enterprises, Inc. and is bound to accept
#    acknowledgement of and abide by the notices and conditions of the
#    Moab Accounting Manager License.
#
#    Copyright (C) 2003 - 2005 Pacific Northwest National Laboratory,
#    Battelle Memorial Institute. All rights reserved.
#
#    The BSD License specifies the terms and conditions for use and
#    redistribution.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#
#    - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
#    - Neither the name of Battelle nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#    POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

package MAM::WSAuthenHandler;

use strict;
use lib qw(/opt/mam/lib /opt/mam/lib/perl5);
use MAM::WS;
use Apache2::Const -compile => qw(OK DECLINED HTTP_UNAUTHORIZED);
use Apache2::Access ();
use Apache2::RequestRec ();
use APR::Table ();
use Sys::Syslog;

openlog('mamws','','local0');

# ----------------------------------------------------------------------------
# $http_status_code = handler($apache2_requestrec)
# ----------------------------------------------------------------------------

# Handle authentication
sub handler
{
    my $r = shift;
    syslog('debug', 'Entering Authen Handler');

    my %headers = %{$r->headers_in()};
    #foreach my $key (sort keys %headers)
    #{
    #    syslog('info', "Header: $key = " . $headers{$key});
    #}

    # We want to support both oauth as well as basic authentication, so 
    # we cannot configure apache to use AuthType Basic.
    # First we check for oauth authentication.

    # Then we check for basic authentication by checking the Authorization 
    # field for the Basic keyword.
    #my $auth_type = $r->auth_type();
    #syslog('info', "auth_type: $auth_type");
    #if ($auth_type eq 'Basic')
    my $authorization = $headers{Authorization};
    if (defined $authorization && $authorization =~ /^Basic/)
    {
        syslog('info', 'Detected Basic Authorization');
        # We can assume that the user and password are set
        my ($status, $password) = $r->get_basic_auth_pw;
        syslog('debug', "Basic authentication detected with status   [$status]");
        syslog('debug', "Basic authentication detected with user     [" . $r->user . "]");
        syslog('debug', "Basic authentication detected with password [$password]");
        return $status;
    }

    # If we get neither authentication type, we will prompt for Basic
    else
    {
      syslog('warning', 'No Auth Type Specified. Prompt for Basic. Sending HTTP_UNAUTHORIZED');
      #my $auth_name = $r->auth_name();
      #$r->err_headers_out->add('WWW-Authenticate', "Basic realm=\"$auth_name\"");
      $r->auth_type('Basic');
      $r->note_basic_auth_failure;
      return Apache2::Const::HTTP_UNAUTHORIZED; # 401
    }
}

1;
