#!/bin/bash
# Run this script after installing, bootstrapping and starting
# Moab Accounting Manager to establish a sample initial setup for
# the fast allocation accounting mode.

# Set the accounting mode and currency precision
if [[ ! -e /opt/mam/etc/mam-server.conf.initsave ]]; then
  cp /opt/mam/etc/mam-server.conf /opt/mam/etc/mam-server.conf.initsave
fi
if [[ ! -e /opt/mam/etc/mam-client.conf.initsave ]]; then
  cp /opt/mam/etc/mam-client.conf /opt/mam/etc/mam-client.conf.initsave
fi
/usr/bin/perl -p -i -e 's/.*accounting\.mode.*/accounting.mode = fast-allocation/' /opt/mam/etc/mam-server.conf /opt/mam/etc/mam-client.conf
/usr/bin/perl -p -i -e 's/.*currency\.precision.*/currency.precision = 2/' /opt/mam/etc/mam-server.conf /opt/mam/etc/mam-client.conf
# mam-server -r can be killed if MAM was started with systemd
mam-server -k
mam-server

# Define charge rates
mam-create-chargerate -n Processors -z 1/h -d "1 credit per processor-hour"
# List charge rates
mam-list-chargerates

# Define organizations (optional)
mam-create-organization -o sciences -d "Sciences College"
mam-create-organization -o arts -d "Arts College"
mam-list-organizations

# Define users
mam-create-user -n "Amy Miller" --phone "(801) 555-1437" --email "amy@hpc.com" -u amy
mam-create-user -n "Robert Taylor" --phone "(801) 555-1474" --email "bob@hpc.com" -u bob
mam-create-user -n "David Jones" --phone "(801) 555-1436" --email "dave@hpc.com" -u dave

# Define accounts, adding users to accounts
mam-create-account -d "Biology Department" -u amy,^bob -a biology -o sciences
mam-create-account -d "Chemistry Department" -u ^amy,dave -a chemistry -o sciences
mam-create-account -d "Film Department" -u bob,^dave -a film -o arts
# List accounts
mam-list-accounts

# Set default accounts for users
mam-modify-user -a chemistry -u amy
mam-modify-user -a biology -u bob
mam-modify-user -a film -u dave
# List users
mam-list-users

# Define funds
mam-create-fund -a biology -n "biology" --default-deposit 5000
mam-create-fund -a chemistry -n "chemistry" --default-deposit -1
mam-create-fund -a film -n "film"
# List funds
mam-list-funds

# Make deposits - creating allocations
mam-deposit -a biology
mam-deposit -a chemistry -z 3000
mam-deposit -L 2000 -a film
# List funds
mam-list-funds
# List allocations
mam-list-allocations

# Check amy's balance
mam-balance -u amy

# Create an automatic allocation renewal event
mam-create-event --fire-command "Fund Reset" -s "2018-01-01" --rearm-period "3 months^"
# List events
mam-list-events

# Calculate Times
duration=300
timeLimit=600
cpuTime=$((12 * duration / 2))
now=`perl -e 'print time'`
startTime=`perl -e "(\\\$sec, \\\$min, \\\$hour, \\\$day, \\\$mon, \\\$year)=(localtime($now))[0..5]; printf(\"%04d-%02d-%02d %02d:%02d:%02d\", \\\$year+1900, \\\$mon+1, \\\$day, \\\$hour, \\\$min, \\\$sec)"`
endTime=`perl -e "(\\\$sec, \\\$min, \\\$hour, \\\$day, \\\$mon, \\\$year)=(localtime($now+$duration))[0..5]; printf(\"%04d-%02d-%02d %02d:%02d:%02d\", \\\$year+1900, \\\$mon+1, \\\$day, \\\$hour, \\\$min, \\\$sec)"`

# Issue charge
mam-charge -J $$ -u amy -g research -a chemistry -c batch -Q normal -m colony -N 1 -P 12 -C $cpuTime -t $duration -s "$startTime" -e "$endTime"
# Check balance - demonstrating impact to balance and lien is gone
mam-balance -u amy
# List funds - demonstrating impact to amount
mam-list-funds -u amy
# List allocations - demonstrating impact to amounts
mam-list-allocations -A -u amy -a chemistry
# List usage records - demonstrating update of charge, status, duration
mam-list-usagerecords

# Issue refund
mam-refund -J $$ 
# Check balance - demonstrating return of credits to allocations
mam-balance -u amy
# List allocations - demonstrating restoration of amounts
mam-list-allocations -A -u amy -a chemistry
# List usage records - demonstrating cancellation of charge
mam-list-usagerecords

# List transactions related with the job
mam-list-transactions -J $$ --full
# List transactions that were part of the charge request
#requestId=`mam-list-transactions -A Charge -J $$ --show RequestId --quiet | tail -1`
#mam-list-transactions -R $requestId --show="Id,Object,Action,Key,Instance,Amount,Fund,Delta,Details"

# Display bank statement
mam-statement -a chemistry
