#!/bin/bash
# Run this script after installing, bootstrapping and starting
# Moab Accounting Manager to establish a sample initial setup for
# the notional charging accounting mode.

# Set the accounting mode and currency precision
if [[ ! -e /opt/mam/etc/mam-server.conf.initsave ]]; then
  cp /opt/mam/etc/mam-server.conf /opt/mam/etc/mam-server.conf.initsave
fi
if [[ ! -e /opt/mam/etc/mam-client.conf.initsave ]]; then
  cp /opt/mam/etc/mam-client.conf /opt/mam/etc/mam-client.conf.initsave
fi
/usr/bin/perl -p -i -e 's/.*accounting\.mode.*/accounting.mode = notional-charging/' /opt/mam/etc/mam-server.conf /opt/mam/etc/mam-client.conf
/usr/bin/perl -p -i -e 's/.*currency\.precision.*/currency.precision = 2/' /opt/mam/etc/mam-server.conf /opt/mam/etc/mam-client.conf
# mam-server -r can be killed if MAM was started with systemd
mam-server -k
mam-server

# Define charge rates
mam-create-chargerate -n Processors -z 1/h -d "1 credit per processor-hour"
# List charge rates
mam-list-chargerates

# Calculate Times
duration=300
timeLimit=600
cpuTime=$((12 * duration / 2))
now=`perl -e 'print time'`
startTime=`perl -e "(\\\$sec, \\\$min, \\\$hour, \\\$day, \\\$mon, \\\$year)=(localtime($now))[0..5]; printf(\"%04d-%02d-%02d %02d:%02d:%02d\", \\\$year+1900, \\\$mon+1, \\\$day, \\\$hour, \\\$min, \\\$sec)"`
endTime=`perl -e "(\\\$sec, \\\$min, \\\$hour, \\\$day, \\\$mon, \\\$year)=(localtime($now+$duration))[0..5]; printf(\"%04d-%02d-%02d %02d:%02d:%02d\", \\\$year+1900, \\\$mon+1, \\\$day, \\\$hour, \\\$min, \\\$sec)"`

# Issue charge
mam-charge -J $$ -u amy -g research -a chemistry -c batch -Q normal -m colony -N 1 -P 12 -C $cpuTime -t $duration -s "$startTime" -e "$endTime"
# List usage records - demonstrating update of charge, status, duration
mam-list-usagerecords

# Issue refund
mam-refund -J $$ 
# List usage records - demonstrating cancellation of charge
mam-list-usagerecords

# List transactions related with the job
mam-list-transactions -J $$ --full
# List transactions that were part of the charge request
#requestId=`mam-list-transactions -A Charge -J $$ --show RequestId --quiet | tail -1`
#mam-list-transactions -R $requestId --show="Id,Object,Action,Key,Instance,Amount,Fund,Delta,Details"

