#! /usr/bin/perl -wT
################################################################################
#
# Balance Frame for Web GUI
#
# File   :  balance.cgi
#
################################################################################
#
#    This file includes parts which are copyrighted by Adaptive Computing
#    Enterprises, Inc., and parts which are copyrighted by Battelle Memorial
#    Institute.
#
#    The terms and conditions for the use and redistribution of these parts
#    are governed by the Moab Accounting Manager License and the BSD License
#    respectively. Refer to the LICENSE file for details.
#
#    Copyright (C) 2006 - 2016 by Adaptive Computing Enterprises, Inc.
#    All Rights Reserved.
#
#    The Moab Accounting Manager License specifies the terms and conditions
#    for use and redistribution.
#
#    The Moab Accounting Manager License applies to the Moab Accounting
#    Manager software offered by Adaptive Computing Enterprises, Inc.
#    By installing or using this software, Licensee accepts a non-exclusive
#    license from Adaptive Computing Enterprises, Inc. and is bound to accept
#    acknowledgement of and abide by the notices and conditions of the
#    Moab Accounting Manager License.
#
#    Copyright (C) 2003 - 2005 Pacific Northwest National Laboratory,
#    Battelle Memorial Institute. All rights reserved.
#
#    The BSD License specifies the terms and conditions for use and
#    redistribution.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#
#    - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
#    - Neither the name of Battelle nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#    POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

use strict;
use lib qw(/opt/mam/lib /opt/mam/lib/perl5);
use CGI qw(:standard);
use CGI::Session;
use XML::LibXML;
use MAM::CGI;
use MAM::Global;

my $cgi        = new CGI;
my $session    = new CGI::Session(undef, $cgi, {Directory => "/tmp"});
my $username   = $session->param("username");
my $password   = $session->param("password");
my $remoteUser = remote_user();
unless (($username && $password) || defined $remoteUser)
{
    print header;
    print start_html(
        -title  => "Session Expired",
        -onLoad => "top.location.replace(\"login.cgi\");"
    );
    print end_html;
    exit(0);
}
my %tokenArgs =
  defined $remoteUser
  ? (actor => $remoteUser, tokenType => $TOKEN_SYMMETRIC)
  : (
    actor      => "$username",
    tokenType  => $TOKEN_PASSWORD,
    tokenName  => "$username",
    tokenValue => "$password"
  );

my $object               = $cgi->param("myobject");
my $action               = $cgi->param("myaction");
my $constraintNames      = $cgi->param("constraintNames") || '';
my $constraintValues     = $cgi->param("constraintValues") || '';
my $display              = $cgi->param("display");
my $currency_enablehours = $clientConfig->{'currency.enablehours'};
my $showHours =
  $cgi->param("ShowHours")
  || (defined $currency_enablehours && $currency_enablehours =~ /true/i)
  ? 'True'
  : 'False';
my $time_division = $showHours eq "True" ? 3600 : 1;
my $currency_precision =
  $showHours eq "True" ? 2 : $clientConfig->{'currency.precision'} || 0;

# Untaint currency precision
if ($currency_precision =~ /^(\d+)$/)
{
    $currency_precision = $1;
}
else
{
    die
      "Illegal characters were found in \$currency_precision ($currency_precision)\n";
}

# Parse out selected constraints
my %selected = ();
@selected{split /,/, $constraintNames} = (split /,/, $constraintValues);

# Javascript to markChange
my $java_script = <<END_JAVA_SCRIPT;
function markChange(name){
  var hiddenNamesInput = document.getElementById("constraintNames");
  var hiddenValuesInput = document.getElementById("constraintValues");
  // alert("before names=" + hiddenNamesInput.value);
  // alert("before values=" + hiddenValuesInput.value);
  var mySelectValue = document.getElementById(name).value;
  
  var nameList = hiddenNamesInput.value.split(",");
  var valueList = hiddenValuesInput.value.split(",");
  if (nameList == "") {
    nameList = [];
    valueList = [];
  }
  for (var i in nameList) {
    if (nameList[i] == name) {
     nameList.splice(i, 1);
     valueList.splice(i, 1);
    }
  }

  // If we have a value, append it back on to the list
  if (mySelectValue != "")
  {
    nameList.push(name);
    valueList.push(mySelectValue);
  }
  hiddenNamesInput.value = nameList.join(',');
  hiddenValuesInput.value = valueList.join(',');
  // alert("after names=" + hiddenNamesInput.value);
  // alert("after values=" + hiddenValuesInput.value);
}
END_JAVA_SCRIPT

print header;
print start_html(
    -title => "Display Balance",
    -script =>
      [{-src => "/cgi-bin/mam/scripts/sorttable.js"}, {-code => $java_script}],
    -style => {-src => "/cgi-bin/mam/styles/statement.css"}
);
print_body();
print end_html;

sub print_body
{
    # Print the header
    my @headerParts = ({-class => "header"});
    push @headerParts, h1("Display Balance");
    print div(@headerParts);

    # Print the prebalance form
    print <<END_OF_SCREEN_TOP;
<form name="balanceForm" method="post" action="balance.cgi">
  <input type="hidden" name="myobject" value="$object">
  <input type="hidden" name="myaction" value="$action">
  <input type="hidden" name="constraintNames" id="constraintNames" value="">
  <input type="hidden" name="constraintValues" id="constraintValues" value="">
  <input type="hidden" name="display" value="True">
END_OF_SCREEN_TOP

    # Build up list of constraints
    my %constraints = ();

    # Constraint Query
    my $request = new MAM::Request(
        object => "Constraint",
        action => "Query",
        %tokenArgs
    );
    $request->addSelection("Name");
    $request->addSelection("Value");
    my $response = $request->getResponse();
    my $status   = $response->getStatus();
    my $message  = $response->getMessage();
    my $data     = $response->getDataElement();
    my $count    = $response->getCount();
    my $doc      = XML::LibXML::Document->new();
    $doc->setDocumentElement($data);

    foreach my $row ($data->childNodes())
    {
        my $name  = ($row->childNodes())[0]->textContent();
        my $value = ($row->childNodes())[1]->textContent();
        $constraints{$name}{$value}++ unless $value =~ /^!/;
    }

    if (scalar keys %constraints)
    {
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
END_OF_SCREEN_SECTION
    }

    foreach my $name (sort keys %constraints)
    {
        # Print Constraint Textbox
        print <<END_OF_SCREEN_SECTION;
    <div class="multiSelect">
      <label class="rowLabel" for="$name">${name}:</label>
      <input type="text" onchange="markChange('$name');" name="$name" id="$name" value="" size="26" maxlength="80"/>
    </div>
END_OF_SCREEN_SECTION
    }

    print "</div>\n" if scalar keys %constraints;

    if (defined $currency_enablehours && $currency_enablehours =~ /true/i)
    {
        print <<END_OF_HTML;
  <div class="row">
    <label class="rowLabel" for="name(./*[1])">Show Hours?</label>
    <input type="radio" name="ShowHours" id="ShowHoursYes" value="True">
    <label for="ShowHoursYes">Yes</label>
    <input type="radio" name="ShowHours" id="ShowHoursNo" value="False" checked="true">
    <label for="ShowHoursNo">No</label>
  </div>
END_OF_HTML
    }

    print <<END_OF_SCREEN_BOTTOM;
  <div id="submitRow">
    <input class="button" type="submit" value="Display Balance">
  </div>
</form>
END_OF_SCREEN_BOTTOM

    if ($display)
    {
        # Fund Query Active:=True UseRules:=True User,Account,Machine:=...
        my $request = new MAM::Request(
            object => "Fund",
            action => "Query",
            %tokenArgs
        );
        $request->addSelection("Id");
        $request->addSelection("Name");
        $request->setOption("Active",           "True");
        $request->setOption("UseRules",         "True");
        $request->setOption("IncludeAncestors", "True");
        foreach my $name (keys %selected)
        {
            $request->addOption("Filter", "${name}=$selected{$name}");
        }
        my $response = $request->getResponse();
        my $status   = $response->getStatus();
        my $message  = $response->getMessage();
        my $data     = $response->getDataElement();
        my $count    = $response->getCount();
        my $doc      = XML::LibXML::Document->new();
        $doc->setDocumentElement($data);

        my @rows = $data->childNodes();
        if (@rows > 0)
        {
            print <<END_OF_TABLE;
    <table class="sortable" id="funds" border="0" cellspacing="0">
      <thead>
      <tr>
        <td colspan="1" rowspan="2">Fund</td>
        <td colspan="1" rowspan="2">Name</td>
        <td colspan="1" rowspan="2">Balance</td>
        <td colspan="1" rowspan="2">Reserved</td>
        <td colspan="1" rowspan="2">Effective</td>
        <td colspan="1" rowspan="2">Credit Limit</td>
        <td colspan="1" rowspan="2">Available</td>
        <td colspan="1" rowspan="2">Allocated</td>
        <td colspan="1" rowspan="2">Percent Used</td>
      </tr>
      <tr>
      </tr>
      </thead>
      <tbody>
END_OF_TABLE

            my %funds            = ();
            my $totalBalance     = 0;
            my $totalReserved    = 0;
            my $totalEffective   = 0;
            my $totalCreditLimit = 0;
            my $totalAvailable   = 0;
            my $totalAllocated   = 0;
            my $id               = 0;
            foreach my $row (@rows)
            {

                foreach my $col ($row->childNodes())
                {
                    my $name  = $col->nodeName();
                    my $value = $col->textContent();
                    if ($name eq "Id") { $id = $value; }
                    $funds{$id}{$name} = $value;
                }
            }

            # Extract amount, credit limit and allocated from allocations
            my %amount      = ();
            my %creditLimit = ();
            my %allocated   = ();
            {
# Allocation Query Show:="Sum(Amount),Sum(CreditLimit),Sum(Allocated),GroupBy(Fund)"
# Active==True Fund==...
                my $request = new MAM::Request(
                    object => "Allocation",
                    action => "Query",
                    %tokenArgs
                );
                $request->addSelection("Fund",        "GroupBy");
                $request->addSelection("Amount",      "Sum");
                $request->addSelection("CreditLimit", "Sum");
                $request->addSelection("Allocated",   "Sum");
                $request->addCondition("Active", "True");
                my $response = $request->getResponse();
                my $status   = $response->getStatus();
                my $message  = $response->getMessage();
                my $data     = $response->getDataElement();
                my $count    = $response->getCount();
                my $doc      = XML::LibXML::Document->new();
                $doc->setDocumentElement($data);

                # Iterate over each row of data
                foreach my $row ($data->childNodes())
                {
                    my $fund =
                      ($row->getChildrenByTagName("Fund"))[0]->textContent();
                    my $amount =
                      ($row->getChildrenByTagName("Amount"))[0]->textContent();
                    my $creditLimit =
                      ($row->getChildrenByTagName("CreditLimit"))[0]
                      ->textContent();
                    my $allocated =
                      ($row->getChildrenByTagName("Allocated"))[0]
                      ->textContent();
                    $amount{$fund}      = $amount;
                    $creditLimit{$fund} = $creditLimit;
                    $allocated{$fund}   = $allocated;
                }
            }

            # Extract reserved amount from lien allocations
            my %reserved = ();
            {
# Lien,LienAllocation Query Show:="Sum(LienAllocation.Amount),GroupBy(LienAllocation.Fund)" Lien.StartTime<=now Lien.EndTime>now Lien.Id==LienAllocation.Lien
                my $request = new MAM::Request(
                    action => "Query",
                    %tokenArgs
                );
                $request->addObject("Lien");
                $request->addObject("LienAllocation");
                $request->addSelection(
                    new MAM::Selection(
                        object => "LienAllocation",
                        name   => "Fund",
                        op     => "GroupBy"
                    )
                );
                $request->addSelection(
                    new MAM::Selection(
                        object => "LienAllocation",
                        name   => "Amount",
                        op     => "Sum"
                    )
                );
                $request->addCondition(
                    new MAM::Condition(
                        object  => "Lien",
                        name    => "Id",
                        subject => "LienAllocation",
                        value   => "Lien"
                    )
                );
                $request->addCondition(
                    new MAM::Condition(
                        object => "Lien",
                        name   => "StartTime",
                        op     => "LE",
                        value  => "now"
                    )
                );
                $request->addCondition(
                    new MAM::Condition(
                        object => "Lien",
                        name   => "EndTime",
                        op     => "GT",
                        value  => "now"
                    )
                );
                my $response = $request->getResponse();
                my $status   = $response->getStatus();
                my $message  = $response->getMessage();
                my $data     = $response->getDataElement();
                my $count    = $response->getCount();
                my $doc      = XML::LibXML::Document->new();
                $doc->setDocumentElement($data);

                # Iterate over each row of data
                foreach my $row ($data->childNodes())
                {
                    my $fund =
                      ($row->getChildrenByTagName("Fund"))[0]->textContent();
                    my $reserved =
                      ($row->getChildrenByTagName("Amount"))[0]->textContent();
                    $reserved{$fund} = $reserved;
                }
            }

            # Iterate over all funds
            foreach my $fund (sort keys %funds)
            {
                my $name     = $funds{$fund}{"Name"};
                my $balance   = defined $amount{$fund} ? $amount{$fund} : 0;
                my $reserved = defined $reserved{$fund} ? $reserved{$fund} : 0;
                my $creditLimit =
                  defined $creditLimit{$fund} ? $creditLimit{$fund} : 0;
                my $allocated =
                  defined $allocated{$fund} ? $allocated{$fund} : 0;
                my $effective   = $balance - $reserved;
                my $available = $effective + $creditLimit;
                my $percentUsed =
                    ($allocated =~ /inf/i || $creditLimit =~ /inf/i) ? 0
                  : ($allocated == 0 && $creditLimit == 0) ? 100
                  :   ($allocated - $balance) * 100 / ($allocated + $creditLimit);
                my $displayBalance =
                  $balance =~ /inf/i
                  ? 'Infinity'
                  : sprintf("%20.${currency_precision}f",
                    $balance / $time_division);
                my $displayReserved = sprintf("%20.${currency_precision}f",
                    $reserved / $time_division);
                my $displayEffective =
                  $effective =~ /inf/i
                  ? 'Infinity'
                  : sprintf("%20.${currency_precision}f",
                    $effective / $time_division);
                my $displayCreditLimit =
                  $creditLimit =~ /inf/i
                  ? 'Infinity'
                  : sprintf("%20.${currency_precision}f",
                    $creditLimit / $time_division);
                my $displayAvailable =
                  $available =~ /inf/i
                  ? 'Infinity'
                  : sprintf("%20.${currency_precision}f",
                    $available / $time_division);
                my $displayAllocated =
                  $allocated =~ /inf/i
                  ? 'Infinity'
                  : sprintf("%20.${currency_precision}f",
                    $allocated / $time_division);
                my $displayPercentUsed = sprintf("%.2f", $percentUsed);
                $totalBalance     += $balance;
                $totalReserved    += $reserved;
                $totalEffective   += $effective;
                $totalCreditLimit += $creditLimit;
                $totalAvailable   += $available;
                $totalAllocated   += $allocated;

                print <<END_OF_TABLE;
      <tr>
        <td>$fund</td>
        <td>$name</td>
        <td align="right" class="amount">$displayBalance</td>
        <td align="right" class="amount">$displayReserved</td>
        <td align="right" class="amount">$displayEffective</td>
        <td align="right" class="amount">$displayCreditLimit</td>
        <td align="right" class="amount">$displayAvailable</td>
        <td align="right" class="amount">$displayAllocated</td>
        <td align="right" class="amount">$displayPercentUsed</td>
      </tr>
END_OF_TABLE
            }

            my $displayTotalBalance =
              $totalBalance =~ /inf/i
              ? 'Infinity'
              : sprintf("%20.${currency_precision}f",
                $totalBalance / $time_division);
            my $displayTotalReserved = sprintf("%20.${currency_precision}f",
                $totalReserved / $time_division);
            my $displayTotalEffective =
              $totalEffective =~ /inf/i
              ? 'Infinity'
              : sprintf("%20.${currency_precision}f",
                $totalEffective / $time_division);
            my $displayTotalCreditLimit =
              $totalCreditLimit =~ /inf/i
              ? 'Infinity'
              : sprintf("%20.${currency_precision}f",
                $totalCreditLimit / $time_division);
            my $displayTotalAvailable =
              $totalAvailable =~ /inf/i
              ? 'Infinity'
              : sprintf("%20.${currency_precision}f",
                $totalAvailable / $time_division);
            my $displayTotalAllocated =
              $totalAllocated =~ /inf/i
              ? 'Infinity'
              : sprintf("%20.${currency_precision}f",
                $totalAllocated / $time_division);
            my $averagePercentUsed =
                ($totalAllocated =~ /inf/i || $totalCreditLimit =~ /inf/i) ? 0
              : ($totalAllocated == 0 && $totalCreditLimit == 0) ? 100
              : ($totalAllocated - $totalBalance) * 100 /
              ($totalAllocated + $totalCreditLimit);
            my $displayAveragePercentUsed =
              sprintf("%.2f", $averagePercentUsed);

            print <<END_OF_TABLE;
      <tr class="totals">
        <td class="Total" colspan="2">Total:</td>
        <td align="right" class="amount">$displayTotalBalance</td>
        <td align="right" class="amount">$displayTotalReserved</td>
        <td align="right" class="amount">$displayTotalEffective</td>
        <td align="right" class="amount">$displayTotalCreditLimit</td>
        <td align="right" class="amount">$displayTotalAvailable</td>
        <td align="right" class="amount">$displayTotalAllocated</td>
        <td align="right" class="amount">$displayAveragePercentUsed</td>
      </tr>
      </tbody>
    </table>
END_OF_TABLE

        }
    }
}

