#! /usr/bin/perl -wT
################################################################################
#
# List Frame for Web GUI
# This is used to list the object and associations in table form
# for actions such as Query, Modify, Delete and Undelete
#
# File   :  list.cgi
#
################################################################################
#
#    This file includes parts which are copyrighted by Adaptive Computing
#    Enterprises, Inc., and parts which are copyrighted by Battelle Memorial
#    Institute.
#
#    The terms and conditions for the use and redistribution of these parts
#    are governed by the Moab Accounting Manager License and the BSD License
#    respectively. Refer to the LICENSE file for details.
#
#    Copyright (C) 2006 - 2016 by Adaptive Computing Enterprises, Inc.
#    All Rights Reserved.
#
#    The Moab Accounting Manager License specifies the terms and conditions
#    for use and redistribution.
#
#    The Moab Accounting Manager License applies to the Moab Accounting
#    Manager software offered by Adaptive Computing Enterprises, Inc.
#    By installing or using this software, Licensee accepts a non-exclusive
#    license from Adaptive Computing Enterprises, Inc. and is bound to accept
#    acknowledgement of and abide by the notices and conditions of the
#    Moab Accounting Manager License.
#
#    Copyright (C) 2003 - 2005 Pacific Northwest National Laboratory,
#    Battelle Memorial Institute. All rights reserved.
#
#    The BSD License specifies the terms and conditions for use and
#    redistribution.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#
#    - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
#    - Neither the name of Battelle nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#    POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

use strict;
use lib qw(/opt/mam/lib /opt/mam/lib/perl5);
use CGI qw(:standard);
use CGI::Session;
use XML::LibXML;
use MAM::CGI;
use MAM::Global;

my $cgi        = new CGI;
my $session    = new CGI::Session(undef, $cgi, {Directory => "/tmp"});
my $remoteUser = remote_user();
my $username   = $remoteUser || $session->param("username");
my $password   = $session->param("password");
unless (($username && $password) || defined $remoteUser)
{
    print header;
    print start_html(
        -title  => "Session Expired",
        -onLoad => "top.location.replace(\"login.cgi\");"
    );
    print end_html;
    exit(0);
}
my %tokenArgs =
  defined $remoteUser
  ? (actor => $remoteUser, tokenType => $TOKEN_SYMMETRIC)
  : (
    actor      => "$username",
    tokenType  => $TOKEN_PASSWORD,
    tokenName  => "$username",
    tokenValue => "$password"
  );

my $priv_ref             = $session->param("priv");
my %priv                 = %{$priv_ref};
my $objects_ref          = $session->param("objects");
my %objects              = %{$objects_ref};
my $actions_ref          = $session->param("actions");
my %actions              = %{$actions_ref};
my $attributes_ref       = $session->param("attributes");
my %attributes           = %{$attributes_ref};
my $object               = $cgi->param("myobject");
my $action               = $cgi->param("myaction");
my $chunk                = $cgi->param("mychunk") || 1;
my $limit                = 1000;
my $instance             = $priv{$object}{$action};
my $conditionNames       = $cgi->param("conditionNames") || '';
my $conditionValues      = $cgi->param("conditionValues") || '';
my $optionNames          = $cgi->param("optionNames") || '';
my $optionValues         = $cgi->param("optionValues") || '';
my $filterNames          = $cgi->param("filterNames") || '';
my $filterValues         = $cgi->param("filterValues") || '';
my $currency_enablehours = $clientConfig->{'currency.enablehours'};
my $showHours =
  $cgi->param("ShowHours")
  || (defined $currency_enablehours && $currency_enablehours =~ /true/i)
  ? 'True'
  : 'False';
my $time_division = $showHours eq "True" ? 3600 : 1;
my $currency_precision =
  $showHours eq "True" ? 2 : $clientConfig->{'currency.precision'} || 0;
my @columnInfo = ();

# Untaint currency precision
if ($currency_precision =~ /^(\d+)$/)
{
    $currency_precision = $1;
}
else
{
    die
      "Illegal characters were found in \$currency_precision ($currency_precision)\n";
}

# Parse out selected conditions and options
my (%conditions, %options, %filters);
@conditions{split /,/, $conditionNames} = (split /,/, $conditionValues);
@options{split /,/,    $optionNames}    = (split /,/, $optionValues);
@filters{split /,/,    $filterNames}    = (split /,/, $filterValues);

# Sort subroutine for association children

my %field_precedence = (
    "Object"       => -17,
    "Type"         => -16,
    "Id"           => -15,
    "Name"         => -14,
    "Active"       => -13,
    "Admin"        => -12,
    "Access"       => -11,
    "Instance"     => -10,
    "DepositShare" => -9,
    "Overflow"     => -8,
    "Rate"         => -7,
    "User"         => -6,
    "Account"      => -5,
    "Machine"      => -4,
    "Fund"         => -3,
    "Amount"       => -2,
    "Value"        => -1,
);

sub by_field
{
    return ((defined($field_precedence{$a}) ? $field_precedence{$a} : 0)
        <=> (defined($field_precedence{$b}) ? $field_precedence{$b} : 0))
      || ($a cmp $b);
}

# Initial Javascript section to define updateStatus for errors
my $java_script = <<END_JAVA_SCRIPT;
  var searchValues = new Array();

  function updateStatus(statusInfo) {
    if (statusInfo)
    {
      parent.mamstatusbar.document.location="status.cgi?" + statusInfo;
    }
  }
END_JAVA_SCRIPT

# Derive primary keys from %attributes
my @primaryKeys = ();
foreach my $name (keys %{$attributes{$object}})
{
    if ($attributes{$object}{$name}{"PrimaryKey"} eq "True")
    {
        push @primaryKeys, $name;
    }
}

# Build the primary object query
# This may have to be revamped to sort on the primary key if there
# turns out to be a problem with Postgres returning nondeterministic
# results when using Limit.
# SELECT results when using Limit
my $request = new MAM::Request(
    object => $object,
    action => "Query",
    %tokenArgs
);
$request->setOption("Limit", $limit);
$request->setOption("Offset", ($chunk - 1) * $limit) if $chunk > 1;
if ($action eq "Undelete")
{
    $request->addCondition("Deleted", "True");
}

# Constrain Query to appropriate privileges
if ($instance eq "ANY")
{
    # Do nothing
}
elsif ($instance eq "SELF")
{
    if ($object eq "User")
    {
        $request->addCondition("Name", "$username");
    }
    elsif (exists $attributes{$object}{"User"})
    {
        $request->addCondition("User", "$username");
    }
}
elsif ($instance eq "ADMIN")
{
    # Find out what instances you are an admin of and include these
}
elsif ($instance eq "MEMBERS")
{
    # Find out what instances you are a member of and include these
}
elsif ($instance eq "NONE")
{
    exit 0;
}
else
{
    $request->addCondition($primaryKeys[0], $instance);
}

# Add Transaction Selections
if ($object eq "Transaction")
{
    $request->addSelection("Id", "Sort");
    $request->addSelection("Object");
    $request->addSelection("Action");
    $request->addSelection("Actor");
    $request->addSelection("Key");
    $request->addSelection("Child");
    $request->addSelection("Instance");
    $request->addSelection("Count");
    $request->addSelection("Amount");
    $request->addSelection("Delta");
    $request->addSelection("Balance");
    $request->addSelection("User");
    $request->addSelection("Account");
    $request->addSelection("Machine");
    $request->addSelection("Fund");
    $request->addSelection("Allocation");
    $request->addSelection("UsageRecord");
    $request->addSelection("Duration");
    $request->addSelection("CreationTime");
}

# Add Conditions from pre-screen
foreach my $name (sort keys %conditions)
{
    if ($name eq "StartTime")
    {
        $request->addCondition($name, $conditions{$name}, "GE");
    }
    elsif ($name eq "EndTime")
    {
        $request->addCondition($name, $conditions{$name}, "LT");
    }
    elsif ($name eq "EarliestCreationTime")
    {
        $request->addCondition("CreationTime", $conditions{$name}, "GE");
    }
    elsif ($name eq "LatestCreationTime")
    {
        $request->addCondition("CreationTime", $conditions{$name}, "LT");
    }
    elsif ($name eq "Instance" || $name eq "Name")
    {
        my $value = $conditions{$name};
        $name =~ s/\*/%/g;
        $name =~ s/\?/_/g;
        $request->addCondition($name, $value, "Match");
    }
    else
    {
        $request->addCondition($name, $conditions{$name});
    }
}

# Add Filters from pre-screen
foreach my $name (sort keys %filters)
{
    my $value = $filters{$name};
    $request->addOption("Filter", "${name}=${value}");
}

# Add Options from pre-screen
foreach my $name (sort keys %options)
{
    $request->addOption($name, $options{$name});
}

# Issue the Query
my $response = $request->getResponse();
my $status   = $response->getStatus();
my $message  = $response->getMessage();
my $data     = $response->getDataElement();
my %metadata = $response->getMetadata();
my $count    = $response->getCount();
my $doc      = XML::LibXML::Document->new();
$doc->setDocumentElement($data);

# On success, add member data to response
if ($status ne "Failure")
{
    my %members;
    my %amount;
    #my %creditLimit;

    # Build up column info for the simple data
    my @rows = $data->childNodes();
    if (@rows > 0)
    {
        my $count = 0;
        foreach my $col ($rows[0]->childNodes())
        {
            my $name = $col->nodeName();
            $columnInfo[$count]{"Name"}   = $name;
            $columnInfo[$count]{"Fields"} = {};
            $count++;
        }
    }

    # Iterate through data if action is Modify, Delete or Undelete
    # Appending a column containing the appropriate pk conditions for the row
    if (   $action eq "Modify"
        || $action eq "Delete"
        || $action eq "Undelete")
    {
        foreach my $row (@rows)
        {
            my @conditions = ();
            my $actionCol  = new XML::LibXML::Element("_${action}");
            foreach my $col ($row->childNodes())
            {
                my $name  = $col->nodeName();
                my $value = $col->textContent();
                if (grep($_ eq $name, @primaryKeys))
                {
                    push @conditions, "$name=$value";
                }
            }
            $actionCol->appendText(join ',', @conditions);
            $row->appendChild($actionCol);
        }
    }

    # Build up a list of associations from %objects
    my @associations = ();
    foreach my $name (sort keys %objects)
    {
        if (   $objects{$name}{"Association"} eq "True"
            && $objects{$name}{"Parent"} eq $object
            && $actions{$name}{"Query"}{"Display"} eq "True")
        {
            push @associations, $name;
        }
    }

    # Iterate over all associations, merging in data
    foreach my $association (sort @associations)
    {
        my $child = $objects{$association}{"Child"} . "s";

        my $request = new MAM::Request(
            object => $association,
            action => "Query",
            %tokenArgs
        );
        if ($action eq "Undelete")
        {
            $request->addCondition("Deleted", "True");
        }
        my $response    = $request->getResponse();
        my $status      = $response->getStatus();
        my $message     = $response->getMessage();
        my $subData     = $response->getDataElement();
        my %subMetadata = $response->getMetadata();
        my $subDoc      = XML::LibXML::Document->new();
        $subDoc->setDocumentElement($subData);

        if ($status ne "Failure")
        {
            my @rows = $subData->childNodes();
            if (@rows > 0)
            {
                # Build up column info for the structured (association) data
                my $newCol = $#columnInfo + 1;
                $columnInfo[$newCol]{"Name"} = $child;

               # Add association data to the hash:
               # $members{$parent}{$child}[$row]{$propertyName} = $propertyValue
                foreach my $row (@rows)
                {
                    my %properties = ();
                    my $parent =
                      ($row->getChildrenByTagName($object))[0]->textContent();
                    foreach my $col ($row->childNodes())
                    {
                        my $name = $col->nodeName();
                        $columnInfo[$newCol]{"Fields"}{$name} ||= 1;
                        next
                          if $name eq $object;    # Exclude parent foreign key
                        my $value = $col->textContent();
                        # This needs to be corrected to include an array of KVPs
                        $properties{$name} = $value;
                    }
                    push @{$members{$parent}{$child}}, \%properties;
                }

                # Add metadata
                foreach my $name (keys %subMetadata)
                {
                    $metadata{$name} = $subMetadata{$name}
                      unless defined $metadata{$name};
                }
            }
        }
        else
        {
            # Just display error message
            print header;
            print start_html(
                -title  => "$PRODUCT Error",
                -script => {-code => $java_script},
                -onLoad => "updateStatus(\"message=$message\")"
            );
            print end_html;
            exit 0;
        }
    }

    # Populate amount info from allocations if object equals fund
    if ($object eq "Fund" && exists $objects{"Allocation"})
    {
        my $request = new MAM::Request(
            object => "Allocation",
            action => "Query",
            %tokenArgs
        );
        if ($action eq "Undelete")
        {
            $request->addCondition("Deleted", "True");
        }
        $request->addSelection("Amount", "Sum");
        #$request->addSelection("CreditLimit", "Sum");
        $request->addSelection("Fund", "GroupBy");
        $request->addCondition("Active", "True");
        my $response    = $request->getResponse();
        my $status      = $response->getStatus();
        my $message     = $response->getMessage();
        my $subData     = $response->getDataElement();
        my %subMetadata = $response->getMetadata();
        my $subDoc      = XML::LibXML::Document->new();
        $subDoc->setDocumentElement($subData);

        if ($status ne "Failure")
        {
            my @rows = $subData->childNodes();
            if (@rows > 0)
            {
                # Build up column info for the allocation amount
                my $amountCol = $#columnInfo + 1;
                $columnInfo[$amountCol]{"Name"}   = "Amount";
                $columnInfo[$amountCol]{"Fields"} = {};
                #my $creditLimitCol = $#columnInfo + 1;
                #$columnInfo[$creditLimitCol]{"Name"}   = "CreditLimit";
                #$columnInfo[$creditLimitCol]{"Fields"} = {};

                # Add amount and creditLimit data to the hash:
                # $amount{$fund} = $amount
                # $creditLimit{$fund} = $creditLimit
                foreach my $row (@rows)
                {
                    my $fund =
                      ($row->getChildrenByTagName("Fund"))[0]->textContent();
                    my $amount =
                      ($row->getChildrenByTagName("Amount"))[0]->textContent();
                    #my $creditLimit =
                    #  ($row->getChildrenByTagName("CreditLimit"))[0]
                    #  ->textContent();
                    $amount{$fund} = $amount;
                    # $creditLimit{$fund} = $creditLimit;
                }

                # Add metadata
                foreach my $name (keys %subMetadata)
                {
                    $metadata{$name} = $subMetadata{$name}
                      unless defined $metadata{$name};
                }
            }
        }
        else
        {
            # Just display error message
            print header;
            print start_html(
                -title  => "$PRODUCT Error",
                -script => {-code => $java_script},
                -onLoad => "updateStatus(\"message=$message\")"
            );
            print end_html;
            exit 0;
        }
    }

    # Populate constraints info from Constraints if object equals Fund
    if ($object eq "Fund" && exists $objects{"Constraint"})
    {
        my $request = new MAM::Request(
            object => "Constraint",
            action => "Query",
            %tokenArgs
        );
        if ($action eq "Undelete")
        {
            $request->addCondition("Deleted", "True");
        }
        $request->addSelection("Fund");
        $request->addSelection("Id");
        $request->addSelection("Name");
        $request->addSelection("Value");
        my $response    = $request->getResponse();
        my $status      = $response->getStatus();
        my $message     = $response->getMessage();
        my $subData     = $response->getDataElement();
        my %subMetadata = $response->getMetadata();
        my $subDoc      = XML::LibXML::Document->new();
        $subDoc->setDocumentElement($subData);

        if ($status ne "Failure")
        {
            my @rows = $subData->childNodes();
            if (@rows > 0)
            {
                # Build up column info for the structured (association) data
                my $newCol = $#columnInfo + 1;
                $columnInfo[$newCol]{"Name"} = "Constraints";

               # Add association data to the hash:
               # $members{$parent}{$child}[$row]{$propertyName} = $propertyValue
                foreach my $row (@rows)
                {
                    my %properties = ();
                    my $parent =
                      ($row->getChildrenByTagName($object))[0]->textContent();
                    foreach my $col ($row->childNodes())
                    {
                        my $name = $col->nodeName();
                        $columnInfo[$newCol]{"Fields"}{$name} ||= 1;
                        next
                          if $name eq $object;    # Exclude parent foreign key
                        my $value = $col->textContent();
                        # This needs to be corrected to include an array of KVPs
                        $properties{$name} = $value;
                    }
                    push @{$members{$parent}{Constraints}}, \%properties;
                }

                # Add metadata
                foreach my $name (keys %subMetadata)
                {
                    $metadata{$name} = $subMetadata{$name}
                      unless defined $metadata{$name};
                }
            }
        }
        else
        {
            # Just display error message
            print header;
            print start_html(
                -title  => "$PRODUCT Error",
                -script => {-code => $java_script},
                -onLoad => "updateStatus(\"message=$message\")"
            );
            print end_html;
            exit 0;
        }
    }

    # Merge member data elements with main data elements into the data element
    # Iterate over each row of data
    if (@primaryKeys)
    {
        foreach my $row ($data->childNodes())
        {
            my $parent =
              ($row->getChildrenByTagName($primaryKeys[0]))[0]->textContent();
            foreach my $name (keys %{$members{$parent}})
            {
                my $newCol = new XML::LibXML::Element($name);
                foreach my $member (@{$members{$parent}{$name}})
                {
                    my $newMember =
                      new XML::LibXML::Element(substr($name, 0, -1));
                    foreach my $field (keys %{$member})
                    {
                        next
                          if $field eq $object;    # Exclude parent foreign key
                        my $newField = new XML::LibXML::Element($field);
                        my $value    = $member->{$field};
                        $newField->appendText($value);
                        $newMember->appendChild($newField);
                    }
                    $newCol->appendChild($newMember);
                }
                $row->appendChild($newCol);
            }
            if ($object eq "Fund" && exists $objects{"Allocation"})
            {
                my $amountCol = new XML::LibXML::Element("Amount");
                if (exists $amount{$parent})
                {
                    $amountCol->appendText($amount{$parent});
                }
                $row->appendChild($amountCol);
                #my $creditLimitCol = new XML::LibXML::Element("CreditLimit");
                #if (exists $creditLimit{$parent})
                #{
                #    $creditLimitCol->appendText($creditLimit{$parent});
                #}
                #$row->appendChild($creditLimitCol);
            }
        }
    }
}
else
{
    # Just display error message
    print header;
    print start_html(
        -title  => "$PRODUCT Error",
        -script => {-code => $java_script},
        -onLoad => "updateStatus(\"message=$message\")"
    );
    print end_html;
    exit 0;
}

# Convert xml data into a table
my $table      = "";
my @fields     = ();
my %values     = ();
my @rows       = $data->childNodes();
my $firstField = "Name";
if (@rows > 0)
{
    # Print Table Header
    $table .= <<END_OF_TABLE_TOP;
<table class="sortable" id="results" border="0" cellspacing="0">
  <thead>
    <tr>
END_OF_TABLE_TOP
    if (   $action eq "Modify"
        || $action eq "Delete"
        || $action eq "Undelete")
    {
        $table .=
          "      <td colspan=\"1\" rowspan=\"2\"><div>Select</div></td>\n";
    }
    my $count = 0;    # Use to distinguish first column name
    foreach my $col (@columnInfo)
    {
        my $name = $col->{"Name"};
        push @fields, $name;
        $firstField = $name unless $count;
        my %fields = %{$col->{"Fields"}};
        if (scalar keys %fields)
        {
            $table .=
                "      <td colspan=\""
              . (scalar(keys(%fields)) - 1)
              . "\" rowspan=\"1\"><div>$name</div></td>\n";
        }
        else
        {
            $table .=
              "      <td colspan=\"1\" rowspan=\"2\"><div>$name</div></td>\n";
        }
        $count++;
    }
    $table .= "    </tr>\n";
    $table .= "    <tr>\n";
    foreach my $col (@columnInfo)
    {
        my $name   = $col->{"Name"};
        my %fields = %{$col->{"Fields"}};
        if (scalar keys %fields)
        {
            foreach my $field (sort by_field keys %fields)
            {
                next
                  if $field eq $object;    # Exclude parent foreign key
                $table .= "       <td><div>$field</div></td>\n";
            }
        }
    }
    $table .= "    </tr>\n  </thead>\n";

    # Print Table Body
    $table .= "  <tbody>\n";
    $count = 0;                            # Reset to count rows
    foreach my $row (@rows)
    {
        $table .=
          "    <tr id=\"row$count\" class=\"t0" . ($count % 2 + 1) . "\">\n";
        if (   $action eq "Modify"
            || $action eq "Delete"
            || $action eq "Undelete")
        {
            my $conditions =
              ($row->getChildrenByTagName("_${action}"))[0]->textContent();
            $table .=
              "      <td><a href=\"#\" onclick=\"document.actionForm.conditions.value=\'$conditions\'; document.actionForm.submit();\">$action</a></td>\n";
        }
        foreach my $col (@columnInfo)
        {
            my $name   = $col->{"Name"};
            my %fields = %{$col->{"Fields"}};
            my $value  = ($row->getChildrenByTagName($name))[0];
            my $text;
            my $alignment = "";
            if (defined $value)
            {
                $text = $value->textContent();
                if ($metadata{$name} eq 'Currency')
                {
                    $alignment = " align=\"right\"";
                    $text      = sprintf("%.${currency_precision}f",
                        $text / $time_division)
                      unless $text eq '';
                    $text =~ s/inf/Infinity/;
                }
                elsif ($metadata{$name} eq 'AutoGen'
                    || $metadata{$name} eq 'Integer'
                    || $metadata{$name} eq 'Float')
                {
                    $alignment = " align=\"right\"";
                }
            }
            else
            {
                $text = "";
            }
            push @{$values{$name}}, $text;
            $text = "&nbsp;"
              if $text eq "";    # So border will be drawn
            if (scalar keys %fields && defined $value)
            {
                my $fieldNum = scalar(keys(%fields)) - 1;
                $table .=
                  "      <td colspan=\"$fieldNum\" valign=\"top\" style=\"padding: 0;\">";
                $table .=
                  "        <table cellspacing=\"0\" border=\"0\" class=\"nested\" style=\"border: 0;\" cols=\"$fieldNum\" width=\"100%\">\n";
                foreach my $member ($value->childNodes())
                {
                    $table .= "          <tr>\n";
                    foreach my $fieldName (sort by_field keys %fields)
                    {
                        next if $fieldName eq $object;
                        my $fieldValue =
                          ($member->getChildrenByTagName($fieldName))[0]
                          ->textContent();
                        if (($fieldName eq "Amount" && $object ne 'ChargeRate')
                            || $fieldName eq "Allocated"
                            || $fieldName eq "Available"
                            || $fieldName eq "Balance"
                            || $fieldName eq "Charge"
                            || $fieldName eq "CreditLimit"
                            || $fieldName eq "Delta"
                            || $fieldName eq "DefaultDeposit"
                            || $fieldName eq "InitialDeposit"
                            || $fieldName eq "Remaining"
                            || $fieldName eq "Reserved")
                        {
                            $alignment  = " align=\"right\"";
                            $fieldValue = sprintf("%.${currency_precision}f",
                                $fieldValue / $time_division)
                              unless $fieldValue eq '';
                            $fieldValue =~ s/inf/Infinity/;
                        }
                        else
                        {
                            $alignment = " align=\"center\"";
                        }
                        $table .=
                          "            <td$alignment style=\"border: 0; font-size: 100%;\">$fieldValue</td>";
                    }
                    $table .= "          </tr>\n";
                }
                $table .= "        </table>\n";
                $table .= "</td>\n";
            }
            else
            {
                $table .=
                    "      <td colspan=\""
                  . (scalar(keys(%fields)) - 1)
                  . "\"$alignment>$text</td>\n";
            }
        }
        $table .= "    </tr>\n";
        $count++;
    }
    $table .= "  </tbody>\n</table>\n";
}

$java_script .= <<END_JAVA_SCRIPT;
  function filterResults() {
    var re = new RegExp(document.filterForm.filter.value, "gi");
    for( i=0; i<searchValues.length; i++) {
      var row = searchValues[i];
      if (row.search(re) != -1) {
        document.getElementById("row" + i).style.display = "";
      } else {
        document.getElementById("row" + i).style.display = "none";
      }
    }    
  }

END_JAVA_SCRIPT
foreach my $field (sort keys %values)
{
    $java_script .= "var ${field}Values = new Array("
      . join(',', map "\"$_\"", @{$values{$field}}) . ");\n";
}
$java_script .= <<END_JAVA_SCRIPT;

function changeSearchField(field) {
    var sortedValues;
    var filterValues = [];
    
END_JAVA_SCRIPT
foreach my $field (sort @fields)
{
    $java_script .= <<END_JAVA_SCRIPT;
    if (field == "$field") {
      searchValues = ${field}Values;
      sortedValues = ${field}Values.slice().sort();
      for( i=0; i<sortedValues.length; i++) {
        if (sortedValues[i] !== sortedValues[i+1] ) {
          filterValues[filterValues.length] = sortedValues[i];
        }
      }    
    }
END_JAVA_SCRIPT
}
$java_script .= <<END_JAVA_SCRIPT;
    new AutoSuggest(document.getElementById('filter'), filterValues);
}

searchValues = ${firstField}Values;
END_JAVA_SCRIPT

# Render the frame
my @statusInfo = ();
if ($message)
{
    $message = join '. ', split /\n/, $message;    # Eliminate carriage returns
    push @statusInfo, "message=$message";
}
my $statusInfo = join '&', @statusInfo;

print header;
my $title;
if   ($action eq "Query") { $title = "List ${object}s"; }
else                      { $title = "$action $object"; }
print start_html(
    -title  => $title,
    -style  => {-src => "/cgi-bin/mam/styles/list.css"},
    -script => [
        {-src  => "/cgi-bin/mam/scripts/autosuggest.js"},
        {-src  => "/cgi-bin/mam/scripts/sorttable.js"},
        {-code => $java_script}
    ],
    -onLoad => "updateStatus(\"$statusInfo\")"
);
print_body();
print end_html;

sub print_body
{
    # Print the header
    my @headerParts = ({-class => "header"});
    push @headerParts, h1($title);
    print div({-id => "autosuggest"}, ul()), div(@headerParts);

    # Print the filter
    print <<END_OF_FILTER_TOP;

<form name="filterForm" onSubmit="filterResults(); return false;" >
  <div class="list">
    <table style="border: 0pt none;">
      <tbody>
        <tr>
          <td>
            <p class="forms">Search:</p>
          </td>
          <td>
            <input id="filter" type="text" name="filter" autocomplete="off">
          </td>
        </tr>
        <tr>
          <td>
            <p class="forms">By:</p>
          </td>
          <td>
            <select name="filter-by" onclick="changeSearchField(this[selectedIndex].text)">
END_OF_FILTER_TOP
    foreach my $field (@fields)
    {
        print "         <option>$field</option>\n";
    }
    # Print the filter
    print <<END_OF_FILTER_BOTTOM;
            </select>
          </td>
        </tr>
      </tbody>
    </table>
  </div>
</form>
END_OF_FILTER_BOTTOM

    # Print the chunk nav if there are more than limit results
    if ($chunk > 1 || $count >= $limit)
    {
        my $begin = ($chunk - 1) * $limit + 1;
        my $end   = ($chunk - 1) * $limit + $count;
        print <<END_OF_FORM;
<div class="table">
  <form name="chunkForm" method="post" action="list.cgi">
    <input type="hidden" name="myobject" value="$object">
    <input type="hidden" name="myaction" value="$action">
END_OF_FORM
        if ($cgi->param("ObjectCondition"))
        {
            print
              "    <input type=\"hidden\" name=\"ObjectCondition\" value=\"",
              $cgi->param("ObjectCondition"), "\">\n";
        }
        if ($cgi->param("ActionCondition"))
        {
            print
              "    <input type=\"hidden\" name=\"ActionCondition\" value=\"",
              $cgi->param("ActionCondition"), "\">\n";
        }
        if ($cgi->param("StartTimeCondition"))
        {
            print
              "    <input type=\"hidden\" name=\"StartTimeCondition\" value=\"",
              $cgi->param("StartTimeCondition"), "\">\n";
        }
        if ($cgi->param("EndTimeCondition"))
        {
            print
              "    <input type=\"hidden\" name=\"EndTimeCondition\" value=\"",
              $cgi->param("EndTimeCondition"), "\">\n";
        }
        if ($cgi->param("InstanceCondition"))
        {
            print
              "    <input type=\"hidden\" name=\"InstanceCondition\" value=\"",
              $cgi->param("InstanceCondition"), "\">\n";
        }
        if ($cgi->param("FundCondition"))
        {
            print
              "    <input type=\"hidden\" name=\"FundCondition\" value=\"",
              $cgi->param("FundCondition"), "\">\n";
        }
        if ($cgi->param("AccountCondition"))
        {
            print
              "    <input type=\"hidden\" name=\"AccountCondition\" value=\"",
              $cgi->param("AccountCondition"), "\">\n";
        }
        if ($cgi->param("UserCondition"))
        {
            print
              "    <input type=\"hidden\" name=\"UserCondition\" value=\"",
              $cgi->param("UserCondition"), "\">\n";
        }
        if ($cgi->param("MachineCondition"))
        {
            print
              "    <input type=\"hidden\" name=\"MachineCondition\" value=\"",
              $cgi->param("MachineCondition"), "\">\n";
        }
        if ($chunk > 1)
        {
            print "<input type=\"submit\" name=\"mychunk\" value=\"",
              $chunk - 1,
              "\" style=\"background:none; background-image: url('/cgi-bin/mam/images/leftbluearrow.gif'); width:24px; height:18px; border:0; cursor:pointer; color:transparent; font-size:0; line-height:0;\" \>\n";
        }
        print "&nbsp;&nbsp;[$begin-$end]&nbsp;&nbsp;";
        if ($count >= $limit)
        {
            print "<input type=\"submit\" name=\"mychunk\" value=\"",
              $chunk + 1,
              "\" style=\"background:none; background-image: url('/cgi-bin/mam/images/rightbluearrow.gif'); width:24px; height:18px; border:0; cursor:pointer; color:transparent; font-size:0; line-height:0;\" \>\n";
        }
        print "\n</form>\n</table>\n";
    }

    # Print the table
    my $lc_action = lc($action);
    $lc_action = "pre" . $lc_action if $lc_action eq "modify";
    print <<END_OF_FORM;
<div class="table">
  <form name="actionForm" method="post" action="${lc_action}.cgi">
    <input type="hidden" name="myobject" value="$object">
    <input type="hidden" name="myaction" value="$action">
    <input type="hidden" name="conditions" value="">
    $table
  </form>
</div>
END_OF_FORM
    #print "<!-- " . $data->toString() . " -->\n";
    print <<END_OF_JAVASCRIPT;
<script language="Javascript">
  new AutoSuggest(document.getElementById('filter'), ${firstField}Values);
</script>
END_OF_JAVASCRIPT
}
