#! /usr/bin/perl -wT
################################################################################
#
# login form
#
# File   :  login.cgi
#
################################################################################
#
#    This file includes parts which are copyrighted by Adaptive Computing
#    Enterprises, Inc., and parts which are copyrighted by Battelle Memorial
#    Institute.
#
#    The terms and conditions for the use and redistribution of these parts
#    are governed by the Moab Accounting Manager License and the BSD License
#    respectively. Refer to the LICENSE file for details.
#
#    Copyright (C) 2006 - 2014 by Adaptive Computing Enterprises, Inc.
#    All Rights Reserved.
#
#    The Moab Accounting Manager License specifies the terms and conditions
#    for use and redistribution.
#
#    The Moab Accounting Manager License applies to the Moab Accounting
#    Manager software offered by Adaptive Computing Enterprises, Inc.
#    By installing or using this software, Licensee accepts a non-exclusive
#    license from Adaptive Computing Enterprises, Inc. and is bound to accept
#    acknowledgement of and abide by the notices and conditions of the
#    Moab Accounting Manager License.
#
#    Copyright (C) 2003 - 2005 Pacific Northwest National Laboratory,
#    Battelle Memorial Institute. All rights reserved.
#
#    The BSD License specifies the terms and conditions for use and
#    redistribution.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#
#    - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
#    - Neither the name of Battelle nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#    POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

use strict;
use lib qw(/opt/mam/lib /opt/mam/lib/perl5);
use CGI qw(:standard);
use CGI::Carp qw(warningsToBrowser fatalsToBrowser);
use CGI::Session;
use MAM::CGI;
use MAM::Global;

my $cgi       = new CGI;
my $username  = $cgi->param("username");
my $password  = $cgi->param("password");

# If this script was not called with username then display login form
if (! $username)
{
    printLoginHeader();
    printLoginForm();
}

# If username was supplied but not a password then redisplay with a message
elsif (! $password)
{
    printLoginHeader();
    print font({-color => "red"}, "You must supply a password");
    printLoginForm();
}

# If there was a taint problem with the userid or password
elsif (my $taintFailure = MAM::CGI::untaint(\$username, \$password))
{
    printLoginHeader();
    print font({-color => "red"}, $taintFailure);
    printLoginForm();
}

# If a username and password were supplied, then test authentication
else
{
    # Test authentication by querying RoleUsers
    my $request = new MAM::Request(
        object     => "RoleUser",
        action     => "Query",
        actor      => "$username",
        tokenType  => $TOKEN_PASSWORD,
        tokenName  => "$username",
        tokenValue => "$password"
    );
    my $response = $request->getResponse();
    my $status   = $response->getStatus();

    # If authentication failed, then redisplay login form with a message
    if ($status eq "Failure")
    {
        printLoginHeader();
        my $message = $response->getMessage();
        if ($message =~ /Failed authenticating message/)
        {
            $message = "You have supplied an invalid username or password.";
        }
        print font({-color => "red"}, $message);
        printLoginForm();
    }

    # If authentication succeeded, start a new session and redirect to welcome
    else
    {
        my $session = new CGI::Session(undef, undef, {Directory => "/tmp"});
        $session->clear();
        $session->expire("+1d");
        $session->param("username", "$username");
        $session->param("password", "$password");
        my $cookie = $cgi->cookie(CGISESSID => $session->id);
        print redirect(-cookie => $cookie, -location => "welcome.cgi");
    }
}

sub printLoginHeader
{
    print header,
      start_html(
        -title => "$PRODUCT Login",
        -style => {-src => "/cgi-bin/mam/styles/login.css"}
      );
}

sub printLoginForm
{
    my $table = table(
        Tr(
            td(
                table(
                    {-border => "0",},
                    Tr(
                        td(
                            table(
                                {
                                    -border      => "0",
                                    -cellpadding => "2",
                                    -cellspacing => "0",
                                    -style       => "font: 10pt;"
                                },
                                Tr(
                                    {-class => "login"},
                                    td(strong("User name:")),
                                    td(
                                        input(
                                            {
                                                -maxlength => "26",
                                                -name      => "username",
                                                -size      => "26",
                                                -type      => "text"
                                            }
                                        )
                                    )
                                ),    # end td
                                Tr(
                                    {-class => "login"},
                                    td(strong("Password:")),
                                    td(
                                        input(
                                            {
                                                -maxlength => "26",
                                                -name      => "password",
                                                -size      => "26",
                                                -type      => "password"
                                            }
                                        )
                                      )    # end td
                                ),         # end Tr
                                Tr(
                                    {-class => "login"},
                                    td(),
                                    td(
                                        {-class => "login"},
                                        input(
                                            {
                                                -type  => "submit",
                                                -value => "Login",
                                                -class => "button"
                                            }
                                        )
                                      )    # end td
                                  )        # end Tr
                              )            # end table
                          )                # end td
                      )                    # end Tr
                  )                        # end table
              )                            # end td
          )                                # end Tr
    );

    local ($") = '';    # Eliminates extra space due to array interpolation
    print div(
        {-id => "title"},
        h2(
            span(
                {
                    -style =>
                      "font-size:36px; font-family:'Arial Black', Gadget, sans-serif;"
                },
                'Moab',
                sup(span({-style => "font-size:10px;"}, '&reg;'))
            ),
            '  Accounting Manager'
        )
      ),
      div(
        {-id => "login"},
        start_form(
            {
                -action  => "login.cgi",
                -enctype => "application/x-www-form-urlencoded",
                -method  => "post"
            }
        ),
        $table,    # end table
        end_form, br,
        p(
            b(
                "Hint: If you do not know your password, use the ",
                i("mam-set-password"),
                " command in a terminal to set or change your password."
            )
        )
      ),
      end_html;
}
