#! /usr/bin/perl -wT
################################################################################
#
# Navigation Menu for MAM Web GUI
#
# File   :  navbar.cgi
#
################################################################################
#
#    This file includes parts which are copyrighted by Adaptive Computing
#    Enterprises, Inc., and parts which are copyrighted by Battelle Memorial
#    Institute.
#
#    The terms and conditions for the use and redistribution of these parts
#    are governed by the Moab Accounting Manager License and the BSD License
#    respectively. Refer to the LICENSE file for details.
#
#    Copyright (C) 2006 - 2016 by Adaptive Computing Enterprises, Inc.
#    All Rights Reserved.
#
#    The Moab Accounting Manager License specifies the terms and conditions
#    for use and redistribution.
#
#    The Moab Accounting Manager License applies to the Moab Accounting
#    Manager software offered by Adaptive Computing Enterprises, Inc.
#    By installing or using this software, Licensee accepts a non-exclusive
#    license from Adaptive Computing Enterprises, Inc. and is bound to accept
#    acknowledgement of and abide by the notices and conditions of the
#    Moab Accounting Manager License.
#
#    Copyright (C) 2003 - 2005 Pacific Northwest National Laboratory,
#    Battelle Memorial Institute. All rights reserved.
#
#    The BSD License specifies the terms and conditions for use and
#    redistribution.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#
#    - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
#    - Neither the name of Battelle nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#    POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

use strict;
use lib qw(/opt/mam/lib /opt/mam/lib/perl5);
use CGI qw(:standard);
use CGI::Session;
use XML::LibXML;
use MAM::CGI;
use MAM::Global;

my $cgi       = new CGI;
my $session   = new CGI::Session(undef, $cgi, {Directory => "/tmp"});
my $isadmin   = $session->param("isadmin");

# Session variables include:
# ==========================
# $username
# $password
# $myobject
# $myaction
# $priv{$object}{$action} = $instance
# $objects{$name}{$propertyName} = $propertyValue
# $actions{$object}{$name}{$propertyName} = $propertyValue
# $attributes{$object}{$name}{$propertyName} = $propertyValue

my %actions    = ();
my %attributes = ();
my %objects    = ();
my %priv       = ();
my %role       = ();

my $remoteUser = remote_user();
my $username   = $remoteUser || $session->param("username");
my $password   = $session->param("password");
my %tokenArgs =
  defined $remoteUser
  ? (actor => $remoteUser, tokenType => $TOKEN_SYMMETRIC)
  : (
    actor      => "$username",
    tokenType  => $TOKEN_PASSWORD,
    tokenName  => "$username",
    tokenValue => "$password"
  );

if (($username && $password) || defined $remoteUser)
{
    my $actions_ref    = $session->param("actions");
    my $attributes_ref = $session->param("attributes");
    my $objects_ref    = $session->param("objects");
    my $priv_ref       = $session->param("priv");
    my $role_ref       = $session->param("role");

    # Determine the objects and actions that should be displayed for this user
    # If the priv session variable is undefined, populate it via a query
    if (defined $priv_ref)
    {
        %priv = %{$priv_ref};
        %role = %{$role_ref};
    }
    else
    {
        my ($request, $response, $status);

# RoleAction,RoleUser Query RoleAction.Role==RoleUser.Role (RoleUser.Name==$username || RoleUser.Name==ANY) Show:="RoleAction.Role,RoleAction.Object,RoleAction.Name,RoleAction.Instance" Unique:=True
        $request = new MAM::Request(
            action => "Query",
            %tokenArgs
        );
        $request->addObject("RoleAction");
        $request->addObject("RoleUser");
        $request->addSelection(
            new MAM::Selection(object => "RoleAction", name => "Role"));
        $request->addSelection(
            new MAM::Selection(object => "RoleAction", name => "Object"));
        $request->addSelection(
            new MAM::Selection(
                object => "RoleAction",
                name   => "Name",
                alias  => "Action"
            )
        );
        $request->addSelection(
            new MAM::Selection(object => "RoleAction", name => "Instance"));
        $request->addCondition(
            new MAM::Condition(
                object  => "RoleAction",
                name    => "Role",
                subject => "RoleUser",
                value   => "Role"
            )
        );
        $request->addCondition(
            new MAM::Condition(
                object => "RoleUser",
                name   => "Name",
                value  => "$username",
                group  => "+1",
                conj   => "And"
            )
        );
        $request->addCondition(
            new MAM::Condition(
                object => "RoleUser",
                name   => "Name",
                value  => "ANY",
                group  => "-1",
                conj   => "Or"
            )
        );
        $request->setOption(
            new MAM::Option(name => "Unique", value => "True"));
        $response = $request->getResponse();
        $status   = $response->getStatus();

        if ($status eq "Success")
        {
            my @data = $response->getData();
            foreach my $datum (@data)
            {
                my $role     = $datum->getPropertyValue("Role");
                my $object   = $datum->getPropertyValue("Object");
                my $action   = $datum->getPropertyValue("Action");
                my $instance = $datum->getPropertyValue("Instance");
                $role_ref->{$object}{$action}{$instance} = $role;

                # Mark whether this user is a system administrator
                if ($role eq "SystemAdmin")
                {
                    $isadmin = "True";
                    $session->param("isadmin", "True");
                }
            }
        }

# Object,Action Query Object.Name==Action.Object Object.Association==False Action.Display==True Show:="Action.Object,Action.Name[as Action]"
        $request = new MAM::Request(
            action => "Query",
            %tokenArgs
        );
        $request->addObject("Object");
        $request->addObject("Action");
        $request->addSelection(
            new MAM::Selection(object => "Action", name => "Object"));
        $request->addSelection(
            new MAM::Selection(
                object => "Action",
                name   => "Name",
                alias  => "Action"
            )
        );
        $request->addCondition(
            new MAM::Condition(
                object  => "Object",
                name    => "Name",
                subject => "Action",
                value   => "Object"
            )
        );
        $request->addCondition(
            new MAM::Condition(
                object => "Object",
                name   => "Association",
                value  => "False"
            )
        );
        $request->addCondition(
            new MAM::Condition(
                object => "Action",
                name   => "Display",
                value  => "True"
            )
        );
        $response = $request->getResponse();
        $status   = $response->getStatus();

        if ($status eq "Success")
        {
            my @data = $response->getData();
            foreach my $datum (@data)
            {
                my $object = $datum->getPropertyValue("Object");
                my $action = $datum->getPropertyValue("Action");
                foreach my $role_object (keys %{$role_ref})
                {
                    next
                      unless ($role_object eq $object || $role_object eq "ANY");
                    foreach my $role_action (keys %{$role_ref->{$role_object}})
                    {
                        next
                          unless ($role_action eq $action
                            || $role_action eq "ANY");
                        foreach my $role_instance (
                            keys %{$role_ref->{$role_object}{$role_action}})
                        {
                            if (
                                ! exists($priv_ref->{$object}{$action})
                                || instance_generality($role_instance) >
                                instance_generality(
                                    $priv_ref->{$object}{$action}
                                )
                              )
                            {
                                $priv_ref->{$object}{$action} = $role_instance;
                            }
                        }
                    }
                }
            }
            $session->param("priv", $priv_ref);
            %priv = %{$priv_ref};
            $session->param("role", $role_ref);
            %role = %{$role_ref};
        }
    }

    # Set Objects Session Variable
    if (defined $objects_ref)
    {
        %objects = %{$objects_ref};
    }
    else
    {
        my ($request, $response, $status);

        # Object Query
        $request = new MAM::Request(
            action => "Query",
            %tokenArgs
        );
        $request->addObject("Object");
        $response = $request->getResponse();
        $status   = $response->getStatus();

        if ($status eq "Success")
        {
            my @data = $response->getData();
            foreach my $datum (@data)
            {
                my $name    = $datum->getPropertyValue("Name");
                my $element = $datum->getElement();
                foreach my $property ($element->childNodes())
                {
                    my $propertyName  = $property->nodeName();
                    my $propertyValue = $property->textContent();
                    $objects_ref->{$name}{$propertyName} = $propertyValue;
                }
            }
            $session->param("objects", $objects_ref);
            %objects = %{$objects_ref};
        }
    }

    # Set Attributes Session Variable
    if (defined $attributes_ref)
    {
        %attributes = %{$attributes_ref};
    }
    else
    {
        my ($request, $response, $status);

        # Attribute Query
        $request = new MAM::Request(
            action => "Query",
            %tokenArgs
        );
        $request->addObject("Attribute");
        $response = $request->getResponse();
        $status   = $response->getStatus();

        if ($status eq "Success")
        {
            my @data = $response->getData();
            foreach my $datum (@data)
            {
                my $object  = $datum->getPropertyValue("Object");
                my $name    = $datum->getPropertyValue("Name");
                my $element = $datum->getElement();
                foreach my $property ($element->childNodes())
                {
                    my $propertyName  = $property->nodeName();
                    my $propertyValue = $property->textContent();
                    $attributes_ref->{$object}{$name}{$propertyName} =
                      $propertyValue;
                }
            }
            $session->param("attributes", $attributes_ref);
            %attributes = %{$attributes_ref};
        }
    }

    # Set Actions Session Variable
    if (defined $actions_ref)
    {
        %actions = %{$actions_ref};
    }
    else
    {
        my ($request, $response, $status);

        # Action Query
        $request = new MAM::Request(
            action => "Query",
            %tokenArgs
        );
        $request->addObject("Action");
        $response = $request->getResponse();
        $status   = $response->getStatus();

        if ($status eq "Success")
        {
            my @data = $response->getData();
            foreach my $datum (@data)
            {
                my $object  = $datum->getPropertyValue("Object");
                my $name    = $datum->getPropertyValue("Name");
                my $element = $datum->getElement();
                foreach my $property ($element->childNodes())
                {
                    my $propertyName  = $property->nodeName();
                    my $propertyValue = $property->textContent();
                    $actions_ref->{$object}{$name}{$propertyName} =
                      $propertyValue;
                }
            }
            $session->param("actions", $actions_ref);
            %actions = %{$actions_ref};
        }
    }
}

my $java_script = "var NavMenus = new Array("
  . join(',', map "\"$_\"", sort keys %priv) . ");\n";
foreach my $object (sort keys %objects)
{
    $java_script .= "var ${object}Actions = new Array("
      . join(',', map "\"$_\"", sort keys %{$actions{$object}}) . ");\n";
}
$java_script .= <<END_JAVA_SCRIPT;
function changeActionCombo(object, form){
    var actionArray;
    var found = false;
END_JAVA_SCRIPT
foreach my $object (sort keys %objects)
{
    $java_script .= <<END_JAVA_SCRIPT;
    if (object == "$object"){
        actionArray = ${object}Actions;
        found = true;
    }
END_JAVA_SCRIPT
}
$java_script .= <<END_JAVA_SCRIPT;
    if(form == 'admin')
        var actionCombo = document.adminChooserForm.myaction;
                                                                        
    for(i=actionCombo.length-1; i >= 0; i--) {
        actionCombo[i] = null;
    }
    if(!found){
        actionCombo[0] = new Option('Select an Action','');
    }
    else{
        actionCombo[0] = new Option('Select an Action','');
        for(i=0; i < actionArray.length; i++) {
            actionCombo[i+1] = new Option(actionArray[i], actionArray[i]);
        }
    }
}
                                                                        
function doManageAction(myobject, myaction, mytitle){
        document.chooserForm.myobject.value = myobject;
        document.chooserForm.myaction.value = myaction;
        myobject = new String(myobject).toLowerCase();
        myaction = new String(myaction).toLowerCase();
                                                                    
        if
        (
          (myaction == 'query') ||
          (myaction == 'modify') ||
          (myaction == 'delete') ||
          (myaction == 'undelete')
        ) {
          if
          (
               (myobject == 'fund') ||
               (myobject == 'allocation') ||
               (myobject == 'charge') ||
               (myobject == 'payment') ||
               (myobject == 'quote') ||
               (myobject == 'lien') ||
               (myobject == 'transaction') ||
               (myobject == 'usagerecord')
          ) {
            document.chooserForm.action="prescreen.cgi";
          }
          else {
            document.chooserForm.action="list.cgi";
          }
        }
        else if (
          (myaction == 'balance') ||
          (myaction == 'statement')
        ) {
          document.chooserForm.action=myaction+".cgi";
        }
        else if (
          (myaction == 'create') ||
          (myaction == 'deposit') ||
          (myaction == 'refund') ||
          (myaction == 'reset') ||
          (myaction == 'transfer') ||
          (myaction == 'withdraw')
        ) {
          document.chooserForm.action="pre"+myaction+".cgi";
        }
        else {
          document.chooserForm.action="unknown.cgi";
        }

        //construct the name of the handling cgi script based on the action
        document.chooserForm.target="action";
        parent.document.title=mytitle;
        document.chooserForm.submit();
}
END_JAVA_SCRIPT

sub print_body
{
    if (keys %objects)
    {
        my $menuArrowClosed = "/cgi-bin/mam/images/menu_arrow_closed.png";

        foreach my $object (sort keys %priv)
        {
            # Don't display Transaction object in viewpoint unless admin
            next if $object eq "Transaction";
            print h3(
                {
                    -class   => "menuHead",
                    -id      => $object,
                    -onclick => "showMenu(\"$object\")"
                },
                img(
                    {
                        -src    => $menuArrowClosed,
                        -class  => "menuArrow",
                        -id     => $object . "Arrow",
                        -width  => "9",
                        -height => "9",
                        -alt    => $object . "s"
                    }
                ),
                $object . "s"
            );
            my $div = "";
            foreach
              my $action (sort actions_by_importance keys %{$priv{$object}})
            {
                my $title = display_action($object, $action);
                $div .= a(
                    {
                        -href =>
                          "javascript:doManageAction(\'$object\', \'$action\', \'$title\')"
                    },
                    $title
                );
                # If we see Fund Query, then also support Fund Balance and 
                # Fund Statement
                # but only if the user has global read perms on Transaction
                if (
                       $object eq "Fund"
                    && $action eq "Query"
                    && (
                        $isadmin
                        || (   exists $role{Transaction}
                            && exists $role{Transaction}{Query}
                            && exists $role{Transaction}{Query}{ANY}
                            && $role{Transaction}{Query}{ANY} eq 'Anonymous')
                    )
                  )
                {
                    $title = display_action($object, "Balance");
                    $div .= a(
                        {
                            -href =>
                              "javascript:doManageAction(\'$object\', \'Balance\', \'$title\')"
                        },
                        $title
                    );
                    $title = display_action($object, "Statement");
                    $div .= a(
                        {
                            -href =>
                              "javascript:doManageAction(\'$object\', \'Statement\', \'$title\')"
                        },
                        $title
                    );
                }
            }
            print div(
                {
                    -class => "menuSection",
                    -id    => $object . "Section"
                },
                $div
            );
        }
        print start_form(
            {
                -name   => "chooserForm",
                -target => "action",
                -method => "post"
            }
          ),
          input(
            {
                -type => "hidden",
                -name => "myobject",
                -id   => "myobject"
            }
          ),
          input(
            {
                -type => "hidden",
                -name => "myaction",
                -id   => "myaction"
            }
          ),
          end_form();
        print br,
          div(
            {-align => "center"},
            start_form(
                {
                    -name   => "logoutForm",
                    -action => "logout.cgi",
                    -target => "action",
                    -method => "post"
                }
            ),
            input(
                {
                    -type  => "submit",
                    -name  => "logout",
                    -value => "Logout",
                    -class => "button"
                }
            ),
            end_form()
          );
    }
}

sub instance_generality
{
    my ($instance) = @_;

    if    ($instance eq "ANY")     { return 4; }
    elsif ($instance eq "MEMBERS") { return 3; }
    elsif ($instance eq "ADMIN")   { return 2; }
    elsif ($instance eq "SELF")    { return 1; }
    else                           { return 1; }
}

my %action_importance = (
    "Create"   => -12,
    "Query"    => -11,
    "Deposit"  => -10,
    "Withdraw" => -9,
    "Transfer" => -8,
    "Charge"   => -7,
    "Reserve"  => -6,
    "Quote"    => -5,
    "Refund"   => -4,
    "Modify"   => -3,
    "Delete"   => -2,
    "Undelete" => -1,
);

sub actions_by_importance
{
    return ((defined($action_importance{$a}) ? $action_importance{$a} : 0)
        <=> (defined($action_importance{$b}) ? $action_importance{$b} : 0))
      || ($a cmp $b);
}

sub display_action
{
    my ($object, $action) = @_;

    if    ($action eq "Query")  { return "List ${object}s"; }
    elsif ($action eq "Create") { return "$action New $object"; }
    elsif ($action eq "Modify"
        || $action eq "Delete"
        || $action eq "Undelete"
        || $action eq "Refund"
        || $action eq "Charge")
    {
        return "$action $object";
    }
    elsif ($action eq "Balance")   { return "Display Balance"; }
    elsif ($action eq "Deposit")   { return "Make Deposit"; }
    elsif ($action eq "Quote")     { return "Issue Quote"; }
    elsif ($action eq "Reserve")   { return "Make Lien"; }
    elsif ($action eq "Statement") { return "Display Statement"; }
    elsif ($action eq "Transfer")  { return "Make Transfer"; }
    elsif ($action eq "Withdraw")  { return "Make Withdrawal"; }
    else                           { return "$action ${object}s"; }
}

print header;
print start_html(
    -title => "MAM Navigation Menu",
    -style => {-src => "/cgi-bin/mam/styles/navbar.css"},
    -script =>
      [{-code => $java_script}, {-src => "/cgi-bin/mam/scripts/menus.js"}],
    -onLoad => 'showMenu("User")'
);
print_body();
print end_html;
