#! /usr/bin/perl -wT
################################################################################
#
# Create Form Frame for Web GUI
#
# File   :  precreate.cgi
#
################################################################################
#
#    This file includes parts which are copyrighted by Adaptive Computing
#    Enterprises, Inc., and parts which are copyrighted by Battelle Memorial
#    Institute.
#
#    The terms and conditions for the use and redistribution of these parts
#    are governed by the Moab Accounting Manager License and the BSD License
#    respectively. Refer to the LICENSE file for details.
#
#    Copyright (C) 2006 - 2016 by Adaptive Computing Enterprises, Inc.
#    All Rights Reserved.
#
#    The Moab Accounting Manager License specifies the terms and conditions
#    for use and redistribution.
#
#    The Moab Accounting Manager License applies to the Moab Accounting
#    Manager software offered by Adaptive Computing Enterprises, Inc.
#    By installing or using this software, Licensee accepts a non-exclusive
#    license from Adaptive Computing Enterprises, Inc. and is bound to accept
#    acknowledgement of and abide by the notices and conditions of the
#    Moab Accounting Manager License.
#
#    Copyright (C) 2003 - 2005 Pacific Northwest National Laboratory,
#    Battelle Memorial Institute. All rights reserved.
#
#    The BSD License specifies the terms and conditions for use and
#    redistribution.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#
#    - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
#    - Neither the name of Battelle nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#    POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

use strict;
use lib qw(/opt/mam/lib /opt/mam/lib/perl5);
use CGI qw(:standard);
use CGI::Session;
use XML::LibXML;
use MAM::CGI;
use MAM::Global;

my $cgi        = new CGI;
my $session    = new CGI::Session(undef, $cgi, {Directory => "/tmp"});
my $username   = $session->param("username");
my $password   = $session->param("password");
my $remoteUser = remote_user();
unless (($username && $password) || defined $remoteUser)
{
    print header;
    print start_html(
        -title  => "Session Expired",
        -onLoad => "top.location.replace(\"login.cgi\");"
    );
    print end_html;
    exit(0);
}
my %tokenArgs =
  defined $remoteUser
  ? (actor => $remoteUser, tokenType => $TOKEN_SYMMETRIC)
  : (
    actor      => "$username",
    tokenType  => $TOKEN_PASSWORD,
    tokenName  => "$username",
    tokenValue => "$password"
  );

my $object         = $cgi->param("myobject");
my $action         = $cgi->param("myaction");
my $objects_ref    = $session->param("objects");
my %objects        = %{$objects_ref};
my $actions_ref    = $session->param("actions");
my %actions        = %{$actions_ref};
my $attributes_ref = $session->param("attributes");
my %attributes     = %{$attributes_ref};

# Javascript to changeActivationTable and getValue
my $java_script = <<END_JAVA_SCRIPT;
function changeActivationTable(type) {
  //for each option, if it is selected make that activation row visible,
  //otherwise make it invisible
  //alert("type: "+ type);
  var selectbox = document.getElementById(type);
                                                                               
  for(i = 0; i != selectbox.options.length; i++) {
    if(selectbox.options[i].value != ""){
      if(selectbox.options[i].selected == true)
        document.getElementById(selectbox.options[i].value+type+"ActivationRow").style.display = "block";
      else if(document.getElementById(selectbox.options[i].value+type+"ActivationRow").style.display == "block")
        document.getElementById(selectbox.options[i].value+type+"ActivationRow").style.display = "none";
    }
  }
}

function getValue(who, type, field){
  if (document.getElementById(who+type+field))
  {
    if(document.getElementById(who+type+field).type == 'checkbox')
    {
      if(document.getElementById(who+type+field).checked)
        return "True";
      else
        return "False";
    }
    else
    {
      return document.getElementById(who+type+field).value;
    }
  }
  else
  {
    return who;
  }
}

counter = 0;
function addCreation(child){
  counter--;
  var oldRow = document.getElementById('Add' + child + 'ActivationRow');
  // Only add the row if the fields are populated
  if (oldRow.getElementsByTagName("div")[2].getElementsByTagName("input")[0].value && oldRow.getElementsByTagName("div")[3].getElementsByTagName("input")[0].value)
  {
    var newRow = oldRow.cloneNode(true);
    newRow.id = counter + child + 'CreationRow';
    newRow.style.display = 'block';
    var divList = newRow.getElementsByTagName("div");
    for (var i=0;i<divList.length;i++) {
      if (i == 0) {
        var newButton = divList[i].getElementsByTagName("input")[0];
        newButton.class = 'button';
        newButton.name = counter + child + 'Delete';
        newButton.id = counter + child + 'Delete';
        newButton.value = 'Delete';
        newButton.onclick = function() { delCreation(newRow.id, child); }
      } else if (i == 1) {
        var txtNode = document.createTextNode("[AutoGen]");
        divList[i].appendChild(txtNode);
      } else if (i > 1) {
        var newField = divList[i].getElementsByTagName("input")[0];
        newField.id = counter + child + newField.name;
        var oldField = oldRow.getElementsByTagName("div")[i].getElementsByTagName("input")[0];
        oldField.value = '';
        addMembers(child,newField.name,newField.value);
      }
    }
    oldRow.parentNode.insertBefore(newRow,oldRow);
  }
}

function delCreation(id, child){
  var newRow = document.getElementById(id);
  var divList = newRow.getElementsByTagName("div");
  for (var i=0;i<divList.length;i++) {
    if (i > 1) {
      var newField = divList[i].getElementsByTagName("input")[0];
      delMembers(child, newField.name, newField.value);
    }
  }
  newRow.parentNode.removeChild(newRow);
}

function addMembers(child, field, value){
  var hiddenInput = document.getElementById("Fund" + child + field);
  if (hiddenInput.value == "")
  {
    hiddenInput.value = value;
  }
  else
  {
    hiddenInput.value = hiddenInput.value + "," + value;
  }
}

function delMembers(child, field, value){
  var hiddenInput = document.getElementById("Fund" + child + field);
  if (hiddenInput.value != "")
  {
    var hiddenInputList = hiddenInput.value.split(",");
    for (var i in hiddenInputList) {
      if (hiddenInputList[i] == value) {
         hiddenInputList.splice(i, 1);
      }
    }
    hiddenInput.value = hiddenInputList.join(',');
  }
}
END_JAVA_SCRIPT

# Print the create form
print header;
print start_html(
    -title  => "Create New $object",
    -script => {-code => $java_script},
    -style  => {-src => "/cgi-bin/mam/styles/filter.css"}
);
print_body();
print end_html;

sub print_body
{
    # Begin dynamic setMembers Javascript function
    my $set_members = <<END_OF_JAVASCRIPT;
function setMembers(type, object){
END_OF_JAVASCRIPT

    # Print the header
    my @headerParts = ({-class => "header"});
    push @headerParts, h1("Create New $object");
    print div(@headerParts);

    # Print symbol key
    print <<END_OF_SCREEN_TOP;
<p id="requiredDesc">Fields marked with a red asterisk (<span class="required">*</span>) are required.</p>
<p id="primaryKeyDesc">Primary key fields are marked with a key icon (<img alt="Primary Key" class="primaryKey" src="/cgi-bin/mam/images/primary_key.gif"/>)</p>
END_OF_SCREEN_TOP

    # Print the create form
    print <<END_OF_SCREEN_TOP;
<form name="inputForm" method="post" action="create.cgi">
  <input type="hidden" name="myobject" value="$object">
  <input type="hidden" name="myaction" value="$action">
END_OF_SCREEN_TOP

    # Print the primary form elements
    foreach my $name (
        sort {
            $attributes{$object}{$a}{Sequence} <=> $attributes{$object}{$b}
              {Sequence}
        } keys %{$attributes{$object}}
      )
    {
        my $hidden       = $attributes{$object}{$name}{Hidden};
        my $primaryKey   = $attributes{$object}{$name}{PrimaryKey};
        my $required     = $attributes{$object}{$name}{Required};
        my $dataType     = $attributes{$object}{$name}{DataType};
        my $values       = $attributes{$object}{$name}{Values};
        my $defaultValue = $attributes{$object}{$name}{DefaultValue};
        my $description  = $attributes{$object}{$name}{Description};

        next if $hidden   eq "True";
        next if $dataType eq "AutoGen";

        # Handle All Data Types
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">
END_OF_SCREEN_SECTION
        if ($primaryKey eq "True")
        {
            print
              "<img alt=\"Primary Key\" class=\"primaryKey\" src=\"/cgi-bin/mam/images/primary_key.gif\"/>";
        }
        if ($required eq "True")
        {
            print "<span class=\"required\">*</span>";
        }
        #print "$description:</label>\n";
        print "$name:</label>\n";

        if ($dataType eq "Boolean")
        {
            print "      <input type=\"radio\" name=\"$name\" value=\"True\"";
            print " CHECKED" if $defaultValue eq "True";
            print ">True\n";
            print "      <input type=\"radio\" name=\"$name\" value=\"False\"";
            print " CHECKED" if $defaultValue eq "False";
            print ">False\n";
        }
        elsif ($values)
        {
            print <<END_OF_SCREEN_SECTION;
    <select id="$name" name="${name}Assignment">
END_OF_SCREEN_SECTION
            print
              "      <Option value=\"$defaultValue\">$defaultValue</Option>\n"
              if defined $defaultValue;

            # Check to see if this is a foreign key
            if ($values =~ /^@(?:[:?!]=)?(\w+)/)
            {
                $values = $1;

         # First figure out what the primary key of the foreign object is called
                my $key;
                foreach my $attribute (keys %{$attributes{$values}})
                {
                    if ($attributes{$values}{$attribute}{"PrimaryKey"} eq
                        "True")
                    {
                        $key ||= $attribute;
                    }
                }
                $key ||= "Name";

                # SELECT $key from $values
                my $request = new MAM::Request(
                    object => $values,
                    action => "Query",
                    %tokenArgs
                );
                $request->addSelection($key);
                my $response = $request->getResponse();
                my $status   = $response->getStatus();
                my $message  = $response->getMessage();
                my $data     = $response->getDataElement();
                my $count    = $response->getCount();
                my $doc      = XML::LibXML::Document->new();
                $doc->setDocumentElement($data);

                # On success, add member data to the select
                if ($status ne "Failure")
                {
                    foreach my $row ($data->childNodes())
                    {
                        my $value = ($row->childNodes())[0]->textContent();
                        print "      <Option value=\"$value\">$value</Option>\n"
                          unless $value eq $defaultValue;
                    }
                }
            }

            # See if it is a member of a list
            elsif ($values =~ /^\((.*)\)$/)
            {
                my @valueList = split /,/, $1;
                foreach my $value (@valueList)
                {
                    print "      <Option value=\"$value\">$value</Option>\n"
                      unless $value eq $defaultValue;
                }
            }

            print "    </select>\n";
        }
        else
        {
            print <<END_OF_SCREEN_SECTION;
    <input maxlength="" size="26" value="$defaultValue" id="$name" name="${name}Assignment" type="text"/>
END_OF_SCREEN_SECTION
        }
        print "  </div>\n";
    }

    # Build up a list of associations from %objects
    my @associations = ();
    foreach my $name (sort keys %objects)
    {
        if (   $objects{$name}{"Association"} eq "True"
            && $objects{$name}{"Parent"} eq $object
            && $actions{$name}{"Query"}{"Display"} eq "True")
        {
            push @associations, $name;
        }
    }

    # Iterate over all associations
    if (@associations)
    {
        print <<END_OF_SCREEN;
<p id="selectInstructions">You can select multiple items from select fields by holding down 'Ctrl' on PCs and 'Cmd' on Macs.</p>
END_OF_SCREEN

        foreach my $association (sort @associations)
        {
            my $child = $objects{$association}{"Child"};

            # Open a setMembers section for each child type
            my $member_object = <<END_OF_JAVASCRIPT;
  if (type == "$child")
  {
END_OF_JAVASCRIPT

            # Build the opening setMembers internal sections
            my $element_bind = <<END_OF_JAVASCRIPT;
    // 1. bind the hidden inputs to the javascript var names
    // hidden inputs named "{object}{child}{field}"
    var nameSelect = document.getElementById(type);
END_OF_JAVASCRIPT

            my $reset_values = <<END_OF_JAVASCRIPT;
    // 2. reset the values of the hidden inputs
END_OF_JAVASCRIPT

            my $select_build = <<END_OF_JAVASCRIPT;
    // 3. loop thru those selected on the list and add to input list
    for(var i = 0; i != nameSelect.options.length; i++){
      if(nameSelect.options[i].selected){
END_OF_JAVASCRIPT

            print <<END_OF_SCREEN;
<div class="row">
<fieldset id="${child}Fieldset">
<legend>${child}s</legend>
<div class="activationTable" id="${child}Activation">
<div class="headRow">
END_OF_SCREEN

            # Build a list of association fields
            my @fields   = ();
            my %dataType = ();
            my %default  = ();
            foreach my $name (
                sort {
                    $attributes{$association}{$a}
                      {Sequence} <=> $attributes{$association}{$b}{Sequence}
                } keys %{$attributes{$association}}
              )
            {
                my $hidden     = $attributes{$association}{$name}{Hidden};
                my $primaryKey = $attributes{$association}{$name}{PrimaryKey};
                my $required   = $attributes{$association}{$name}{Required};
                my $dataType   = $attributes{$association}{$name}{DataType};
                my $values     = $attributes{$association}{$name}{Values};
                my $defaultValue =
                  $attributes{$association}{$name}{DefaultValue};
                my $description = $attributes{$association}{$name}{Description};

                next if $hidden eq "True";
                next if $name   eq $object;    # We don't want the parent key
                push @fields, $name;

                # Build the core setMembers internal sections
                $element_bind .= <<END_OF_JAVASCRIPT;
    var hiddenElement${name} = document.getElementById(object+nameSelect.name+"$name");
END_OF_JAVASCRIPT

                $reset_values .= <<END_OF_JAVASCRIPT;
    hiddenElement${name}.value = "";
END_OF_JAVASCRIPT

                $select_build .= <<END_OF_JAVASCRIPT;
        if (hiddenElement${name}.value == ""){
          hiddenElement${name}.value = hiddenElement${name}.value + getValue(nameSelect.options[i].value, type, "$name");
        }
        else {
          hiddenElement${name}.value = hiddenElement${name}.value + "," + getValue(nameSelect.options[i].value, type, "$name");
        }
END_OF_JAVASCRIPT

                $dataType{$name} = $dataType;
                $default{$name}  = $defaultValue;
                if ($primaryKey eq "True")
                {
                    print "<div class=\"primaryKeyActivation\">$name</div>\n";
                }
                else
                {
                    print "<div class=\"active\">$name</div>\n";
                }
            }
            print "</div>\n";

            # Build list of children to select from
            my $key = "";
            foreach my $name (
                sort {
                    $attributes{$child}{$a}{Sequence} <=> $attributes{$child}
                      {$b}{Sequence}
                } keys %{$attributes{$child}}
              )
            {
                my $hidden       = $attributes{$child}{$name}{Hidden};
                my $primaryKey   = $attributes{$child}{$name}{PrimaryKey};
                my $required     = $attributes{$child}{$name}{Required};
                my $dataType     = $attributes{$child}{$name}{DataType};
                my $values       = $attributes{$child}{$name}{Values};
                my $defaultValue = $attributes{$child}{$name}{DefaultValue};
                my $description  = $attributes{$child}{$name}{Description};

                next if $hidden eq "True";
                if ($primaryKey eq "True")
                {
                    $key ||= $name;
                }
            }

            # Build and issue the associated Request
            # SELECT @names FROM $child
            my $request = new MAM::Request(
                object => $child,
                action => "Query",
                %tokenArgs
            );
            $request->addSelection($key);
            my $response = $request->getResponse();
            my $status   = $response->getStatus();
            my $message  = $response->getMessage();
            my $data     = $response->getDataElement();
            my $count    = $response->getCount();
            my $doc      = XML::LibXML::Document->new();
            $doc->setDocumentElement($data);

            # On success, add member data to the select
            if ($status ne "Failure")
            {
                my @members = ();
                foreach my $row ($data->childNodes())
                {
                    my $name  = ($row->childNodes())[0]->nodeName();
                    my $value = ($row->childNodes())[0]->textContent();
                    foreach my $field (@fields)
                    {
                        # We assume the primary key will always be listed first
                        if ($field eq $key)
                        {
                            push @members, $value;
                            print <<END_OF_SCREEN;
<div id="${value}${child}ActivationRow" class="activationRow">
<div class="primaryKeyActivation">$value</div>
END_OF_SCREEN
                        }
                        else
                        {
                            print "<div class=\"active\">\n";
                            if ($dataType{$field} eq "Boolean")
                            {
                                my $checked = "";
                                if ($default{$field} eq "True")
                                {
                                    $checked = " CHECKED";
                                }
                                print
                                  "<input onchange=\"setMembers(\'$child\', \'$object\');\" id=\"${value}${child}${field}\" name=\"${value}${child}${field}\" value=\"True\" type=\"checkbox\"$checked/>\n";
                            }
                            else
                            {
                                print
                                  "<input type=\"text\" onchange=\"setMembers(\'$child\', \'$object\');\" name=\"${value}${child}${field}\" id=\"${value}${child}${field}\" value=\"$default{$field}\" size=\"10\" maxlength=\"80\"/>\n";
                            }
                            print "</div>\n";
                        }
                    }
                    print "</div>\n";
                }
                print "</div>\n";

                my $disabled = "";
                print <<END_OF_SCREEN;
<div id="${child}MultiSelect" class="multiSelect">
<label for="$child">${child}s</label>
<select multiple="yes" size="10" id="$child" name="$child" onchange="changeActivationTable(\'$child\'); setMembers(\'$child\', \'$object\');"$disabled>
END_OF_SCREEN
                foreach my $member (@members)
                {
                    print "<option value=\"$member\">$member</option>\n";
                }
                print "</select>\n</div>\n";
            }
            foreach my $field (@fields)
            {
                print
                  "<input id=\"${object}${child}${field}\" name=\"${object}${child}${field}\" type=\"hidden\">\n";
            }
            print "</fieldset>\n</div>\n";

            # Build the closing setMembers internal sections
            $select_build .= <<END_OF_JAVASCRIPT;
      }
    }
END_OF_JAVASCRIPT

            # Close a setMembers section for each child type
            $member_object .= <<END_OF_JAVASCRIPT;

$element_bind

$reset_values

$select_build
  }
END_OF_JAVASCRIPT

            # Add member_object to $set_members
            $set_members .= <<END_OF_JAVASCRIPT;

$member_object
END_OF_JAVASCRIPT
        }
    }

    # Handle Fund Constraints
    if ($object eq "Fund")
    {
        print <<END_OF_SCREEN;
<p id="selectInstructions">New constraints will not be considered for addition until after you press the Add button.</p>
END_OF_SCREEN

        my $child       = "Constraint";
        my $association = "Constraint";

        print <<END_OF_SCREEN;
<div class="row">
<fieldset id="${child}Fieldset">
<legend>${child}s</legend>
<div id="${child}Activation">
<div class="headRow">
END_OF_SCREEN

        # Print no header in button column
        print "<div class=\"active\"></div>\n";

        # Build a list of association fields
        my @fields   = ();
        my %dataType = ();
        my %default  = ();
        foreach my $name (
            sort {
                $attributes{$association}{$a}
                  {Sequence} <=> $attributes{$association}{$b}{Sequence}
            } keys %{$attributes{$association}}
          )
        {
            my $hidden       = $attributes{$association}{$name}{Hidden};
            my $primaryKey   = $attributes{$association}{$name}{PrimaryKey};
            my $required     = $attributes{$association}{$name}{Required};
            my $dataType     = $attributes{$association}{$name}{DataType};
            my $values       = $attributes{$association}{$name}{Values};
            my $defaultValue = $attributes{$association}{$name}{DefaultValue};
            my $description  = $attributes{$association}{$name}{Description};

            next if $hidden eq "True";
            next if $name   eq $object;    # We don't want the parent key
            push @fields, $name;

            $dataType{$name} = $dataType;
            $default{$name}  = $defaultValue;
            print "<div class=\"active\">$name</div>\n";
        }
        print "</div>\n";

        # Obtain the child primary key
        my $key = "Id";

        # Print an add row
        print <<END_OF_SCREEN;
<div style="display:block;" id="Add${child}ActivationRow" class="activationRow">
<div class="active">
<input class="button" onClick="addCreation('$child');" id="Add${child}Row" name="Add${child}Row" value="Add" type="button"/>
</div>
END_OF_SCREEN
        foreach my $field (@fields)
        {
            # We assume the primary key will always be listed first
            if ($field eq $key)
            {
                print "<div class=\"active\"></div>\n";
            }
            else
            {
                print "<div class=\"active\">\n";
                if ($dataType{$field} eq "Boolean")
                {
                    my $checked = "";
                    if ($default{$field} eq "True")
                    {
                        $checked = " CHECKED";
                    }
                    print
                      "<input id=\"Add${child}${field}\" name=\"${field}\" value=\"True\" type=\"checkbox\"$checked/>\n";
                }
                else
                {
                    print
                      "<input class=\"${field}\" type=\"text\" name=\"${field}\" id=\"Add${child}${field}\" value=\"$default{$field}\" size=\"10\" maxlength=\"80\"/>\n";
                }
                print "</div>\n";
            }
        }
        print "</div>\n";

        foreach my $field (@fields)
        {
            print
              "<input id=\"${object}${child}${field}\" name=\"${object}${child}${field}\" type=\"hidden\">\n";
        }
        print "</fieldset>\n</div>\n";
    }

    print <<END_OF_SCREEN_BOTTOM;
  <div id="submitRow">
    <input class="button" type="submit" value="Create New $object">
  </div>
</form>
END_OF_SCREEN_BOTTOM

    # End dynamic setMembers Javascript function
    $set_members .= <<END_OF_JAVASCRIPT;
}
END_OF_JAVASCRIPT

    print <<END_OF_JAVASCRIPT;
<script language="Javascript">
$set_members
</script>
END_OF_JAVASCRIPT
}

