#! /usr/bin/perl -wT
################################################################################
#
# Modify Form Frame for Web GUI
#
# File   :  premodify.cgi
#
################################################################################
#
#    This file includes parts which are copyrighted by Adaptive Computing
#    Enterprises, Inc., and parts which are copyrighted by Battelle Memorial
#    Institute.
#
#    The terms and conditions for the use and redistribution of these parts
#    are governed by the Moab Accounting Manager License and the BSD License
#    respectively. Refer to the LICENSE file for details.
#
#    Copyright (C) 2006 - 2016 by Adaptive Computing Enterprises, Inc.
#    All Rights Reserved.
#
#    The Moab Accounting Manager License specifies the terms and conditions
#    for use and redistribution.
#
#    The Moab Accounting Manager License applies to the Moab Accounting
#    Manager software offered by Adaptive Computing Enterprises, Inc.
#    By installing or using this software, Licensee accepts a non-exclusive
#    license from Adaptive Computing Enterprises, Inc. and is bound to accept
#    acknowledgement of and abide by the notices and conditions of the
#    Moab Accounting Manager License.
#
#    Copyright (C) 2003 - 2005 Pacific Northwest National Laboratory,
#    Battelle Memorial Institute. All rights reserved.
#
#    The BSD License specifies the terms and conditions for use and
#    redistribution.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#
#    - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
#    - Neither the name of Battelle nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#    POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

use strict;
use lib qw(/opt/mam/lib /opt/mam/lib/perl5);
use CGI qw(:standard);
use CGI::Session;
use XML::LibXML;
use MAM::CGI;
use MAM::Global;

my $cgi        = new CGI;
my $session    = new CGI::Session(undef, $cgi, {Directory => "/tmp"});
my $username   = $session->param("username");
my $password   = $session->param("password");
my $remoteUser = remote_user();
unless (($username && $password) || defined $remoteUser)
{
    print header;
    print start_html(
        -title  => "Session Expired",
        -onLoad => "top.location.replace(\"login.cgi\");"
    );
    print end_html;
    exit(0);
}
my %tokenArgs =
  defined $remoteUser
  ? (actor => $remoteUser, tokenType => $TOKEN_SYMMETRIC)
  : (
    actor      => "$username",
    tokenType  => $TOKEN_PASSWORD,
    tokenName  => "$username",
    tokenValue => "$password"
  );

my $object         = $cgi->param("myobject");
my $action         = $cgi->param("myaction");
my $conditions     = $cgi->param("conditions");
my $objects_ref    = $session->param("objects");
my %objects        = %{$objects_ref};
my $actions_ref    = $session->param("actions");
my %actions        = %{$actions_ref};
my $attributes_ref = $session->param("attributes");
my %attributes     = %{$attributes_ref};

# Javascript to changeActivationTable
my $java_script = <<END_JAVA_SCRIPT;
function changeActivationTable(child) {
  //for each option, if it is selected make that activation row visible,
  //otherwise make it invisible
  //Also builds up the deleted and created input members
  //alert("child: "+ child);
  var selectbox = document.getElementById(child);
  var creations = document.getElementById(child + "Creations");
  var deletions = document.getElementById(child + "Deletions");
                                                                               
  // Reset the creations and deletions inputs
  creations.value = "";
  deletions.value = "";

  for(i = 0; i != selectbox.options.length; i++)
  {
    if(selectbox.options[i].value != "")
    {
      if(selectbox.options[i].selected == true)
      {
        document.getElementById(selectbox.options[i].value+child+"ActivationRow").style.display = "block";
        var found = false;
        for (j = 0; j != originalArray.length; j++)
        {
          if (child+selectbox.options[i].value == originalArray[j])
          {
            found = true;
          }
        }
        if (found == false)
        {
          // Add to created
          if (creations.value == "")
          {
            creations.value = selectbox.options[i].value;
          }
          else
          {
            creations.value = creations.value + "," + selectbox.options[i].value;
          }
        }
      }
      else
      {
        if(document.getElementById(selectbox.options[i].value+child+"ActivationRow").style.display == "block")
        {
          document.getElementById(selectbox.options[i].value+child+"ActivationRow").style.display = "none";
        }
        var found = false;
        for (j = 0; j != originalArray.length; j++)
        {
          if (child+selectbox.options[i].value == originalArray[j])
          {
            found = true;
          }
        }
        if (found == true)
        {
          // Add to deleted
          if (deletions.value == "")
          {
            deletions.value = selectbox.options[i].value;
          }
          else
          {
            deletions.value = deletions.value + "," + selectbox.options[i].value;
          }
        }
      }
    }
  }
}

function markChange(name){
  var changes = document.getElementById("changes");
  if (changes.value == "")
  {
    changes.value = name;
  }
  else
  {
    changes.value = changes.value + "," + name;
  }
}

function markCreation(value, child){
  var creations = document.getElementById(child + "Creations");
  if (creations.value == "")
  {
    creations.value = value;
  }
  else
  {
    creations.value = creations.value + "," + value;
  }
}

function unmarkCreation(value, child){
  var creations = document.getElementById(child + "Creations");
  if (creations.value != "")
  {
    var creationList = creations.value.split(",");
    for (var i in creationList) {
      if (creationList[i] == value) {
         creationList.splice(i, 1);
      }
    }
    creations.value = creationList.join(',');
  }
}

function markModification(value, child){
  var modifications = document.getElementById(child + "Modifications");
  if (modifications.value == "")
  {
    modifications.value = value;
  }
  else
  {
    modifications.value = modifications.value + "," + value;
  }
}

function markDeletion(value, child){
  var deletions = document.getElementById(child + "Deletions");
  if (deletions.value == "")
  {
    deletions.value = value;
  }
  else
  {
    deletions.value = deletions.value + "," + value;
  }
}

counter = 0;
function addCreation(child){
  counter--;
  var oldRow = document.getElementById('Add' + child + 'ActivationRow');
  // Only add the row if the fields are populated
  if (oldRow.getElementsByTagName("div")[2].getElementsByTagName("input")[0].value && oldRow.getElementsByTagName("div")[3].getElementsByTagName("input")[0].value)
  {
    var newRow = oldRow.cloneNode(true);
    newRow.id = counter + child + 'CreationRow';
    newRow.style.display = 'block';
    var divList = newRow.getElementsByTagName("div");
    for (var i=0;i<divList.length;i++) {
      if (i == 0) {
        var newButton = divList[i].getElementsByTagName("input")[0];
        newButton.name = counter + child + 'Delete';
        newButton.id = counter + child + 'Delete';
        newButton.value = 'Delete';
        newButton.onclick = function() { delCreation(newRow.id); unmarkCreation(counter,child); }
      } else if (i == 1) {
        var txtNode = document.createTextNode("[AutoGen]");
        divList[i].appendChild(txtNode);
      } else if (i > 1) {
        var newField = divList[i].getElementsByTagName("input")[0];
        newField.id = counter + child + newField.name;
        newField.name = counter + child + newField.name;
        var oldField = oldRow.getElementsByTagName("div")[i].getElementsByTagName("input")[0];
        oldField.value = '';
      }
    }
    oldRow.parentNode.insertBefore(newRow,oldRow);
    markCreation(counter,child);
  }
}

function delCreation(id){
  var targetNode = document.getElementById(id);
  targetNode.parentNode.removeChild(targetNode);
}
END_JAVA_SCRIPT

# Print the modify form
print header;
print start_html(
    -title  => "Modify $object",
    -script => {-code => $java_script},
    -style  => {-src => "/cgi-bin/mam/styles/filter.css"},
);
print_body();
print end_html;

sub print_body
{
    # Print the header
    my @headerParts = ({-class => "header"});
    push @headerParts, h1("Modify $object");
    print div(@headerParts);

    # Print symbol key
    print <<END_OF_SCREEN_TOP;
<p id="primaryKeyDesc">Primary key fields are marked with a key icon (<img alt="Primary Key" class="primaryKey" src="/cgi-bin/mam/images/primary_key.gif"/>)</p>
END_OF_SCREEN_TOP

    # Print the modify form
    print <<END_OF_SCREEN_TOP;
<form name="premodifyForm" method="post" action="modify.cgi">
  <input type="hidden" name="myobject" value="$object">
  <input type="hidden" name="myaction" value="$action">
  <input type="hidden" name="conditions" value="$conditions">
  <input type="hidden" name="changes" id="changes" value="">
  <input type="hidden" name="creations" id="creations" value="">
  <input type="hidden" name="deletions" id="deletions" value="">
  <input type="hidden" name="modifications" id="modifications" value="">
END_OF_SCREEN_TOP

    # Print the primary form elements

    # First figure out what the primary key of the foreign object is called
    my @names        = ();
    my %primaryKeys  = ();
    my %fixed        = ();
    my %dataTypes    = ();
    my %values       = ();
    my %descriptions = ();
    foreach my $name (
        sort {
            $attributes{$object}{$a}{Sequence} <=> $attributes{$object}{$b}
              {Sequence}
        } keys %{$attributes{$object}}
      )
    {
        my $hidden       = $attributes{$object}{$name}{Hidden};
        my $primaryKey   = $attributes{$object}{$name}{PrimaryKey};
        my $fixed        = $attributes{$object}{$name}{Fixed};
        my $dataType     = $attributes{$object}{$name}{DataType};
        my $values       = $attributes{$object}{$name}{Values};
        my $defaultValue = $attributes{$object}{$name}{DefaultValue};
        my $description  = $attributes{$object}{$name}{Description};

        next if $hidden eq "True";
        push @names, $name;
        $primaryKeys{$name}  = $primaryKey;
        $fixed{$name}        = $fixed;
        $dataTypes{$name}    = $dataType;
        $values{$name}       = $values;
        $descriptions{$name} = $description;
    }

    # SELECT @names from $object where @conditions
    my $request = new MAM::Request(
        object => $object,
        action => "Query",
        %tokenArgs
    );
    foreach my $name (@names)
    {
        $request->addSelection($name);
    }
    foreach my $condition (split /,/, $conditions)
    {
        my ($name, $value) = split /=/, $condition;
        $request->addCondition($name, $value);
    }
    my $response = $request->getResponse();
    my $status   = $response->getStatus();
    my $message  = $response->getMessage();
    my $data     = $response->getDataElement();
    my $count    = $response->getCount();
    my $doc      = XML::LibXML::Document->new();
    $doc->setDocumentElement($data);

    # On success, add member data to the values hash
    my @rows = $data->childNodes();
    if (@rows > 0)
    {
        foreach my $col ($rows[0]->childNodes())
        {
            my $name        = $col->nodeName();
            my $value       = $col->textContent();
            my $primaryKey  = $primaryKeys{$name};
            my $dataType    = $dataTypes{$name};
            my $values      = $values{$name};
            my $description = $descriptions{$name};

            # Handle All Data Types
            print <<END_OF_SCREEN_SECTION;
    <div class="row">
      <label class="rowLabel" for="$name">
END_OF_SCREEN_SECTION
            if ($primaryKey eq "True")
            {
                print
                  "<img alt=\"Primary Key\" class=\"primaryKey\" src=\"/cgi-bin/mam/images/primary_key.gif\"/>";
            }
            print "$description:</label>\n";

            if (   $fixed{$name} eq "True"
                || ($object eq "Allocation" && $name eq "Amount")
                || ($object eq "Allocation" && $name eq "Active")
                || $name eq "UsageRecord"
                || $name eq "Fund"
                || $name eq "Allocation"
                || $name eq "Lien"
                || $name eq "Quote")
            {
                print <<END_OF_SCREEN_SECTION;
      <input disabled="y" maxlength="" size="26" value="$value" id="$name" name="${name}Assignment" type="text"/>
END_OF_SCREEN_SECTION
            }
            elsif ($dataType eq "Boolean")
            {
                print
                  "      <input onchange=\"markChange(\'${name}Assignment\');\" type=\"radio\" name=\"${name}Assignment\" id=\"$name\" value=\"True\"";
                print " CHECKED" if $value eq "True";
                print ">True\n";
                print
                  "      <input onchange=\"markChange(\'${name}Assignment\');\" type=\"radio\" name=\"${name}Assignment\" id=\"$name\" value=\"False\"";
                print " CHECKED" if $value eq "False";
                print ">False\n";
            }
            elsif ($values)
            {
                print <<END_OF_SCREEN_SECTION;
      <select onchange=\"markChange(\'${name}Assignment\');\" id="$name" name="${name}Assignment">
END_OF_SCREEN_SECTION
                print "      <Option value=\"$value\">$value</Option>\n";

                # Check to see if this is a foreign key
                if ($values =~ /^@(?:[:?!]=)?(\w+)/)
                {
                    $values = $1;

         # First figure out what the primary key of the foreign object is called
                    my $key;
                    foreach my $attribute (keys %{$attributes{$values}})
                    {
                        if ($attributes{$values}{$attribute}{"PrimaryKey"} eq
                            "True")
                        {
                            $key ||= $attribute;
                        }
                    }
                    $key ||= "Name";

                    # SELECT $key from $values
                    my $request = new MAM::Request(
                        object => $values,
                        action => "Query",
                        %tokenArgs
                    );
                    $request->addSelection($key);
                    my $response = $request->getResponse();
                    my $status   = $response->getStatus();
                    my $message  = $response->getMessage();
                    my $data     = $response->getDataElement();
                    my $count    = $response->getCount();
                    my $doc      = XML::LibXML::Document->new();
                    $doc->setDocumentElement($data);

                    # On success, add member data to the select
                    if ($status ne "Failure")
                    {
                        foreach my $row ($data->childNodes())
                        {
                            my $val = ($row->childNodes())[0]->textContent();
                            print "      <Option value=\"$val\">$val</Option>\n"
                              unless $val eq $value;
                        }
                    }
                }

                # See if it is a member of a list
                elsif ($values =~ /^\((.*)\)$/)
                {
                    my @valueList = split /,/, $1;
                    foreach my $val (@valueList)
                    {
                        print "      <Option value=\"$val\">$val</Option>\n"
                          unless $val eq $value;
                    }
                }

                print "    </select>\n";
            }
            else
            {
                print <<END_OF_SCREEN_SECTION;
      <input onchange=\"markChange(\'${name}Assignment\');\" maxlength="" size="26" value="$value" id="$name" name="${name}Assignment" type="text"/>
END_OF_SCREEN_SECTION
            }
            print "  </div>\n";
        }
    }

    # Build up a list of associations from %objects
    my @associations = ();
    my @originals    = ();
    foreach my $name (sort keys %objects)
    {
        if (   $objects{$name}{"Association"} eq "True"
            && $objects{$name}{"Parent"} eq $object
            && $actions{$name}{"Query"}{"Display"} eq "True")
        {
            push @associations, $name;
        }
    }

    # Iterate over all associations
    if (@associations)
    {
        print <<END_OF_SCREEN;
<p id="selectInstructions">You can select multiple items from select fields by holding down 'Ctrl' on PCs and 'Cmd' on Macs.</p>
END_OF_SCREEN

        foreach my $association (sort @associations)
        {
            my $child = $objects{$association}{"Child"};

            print <<END_OF_SCREEN;
  <input type="hidden" name="${child}Creations" id="${child}Creations" value="">
  <input type="hidden" name="${child}Deletions" id="${child}Deletions" value="">
  <input type="hidden" name="${child}Modifications" id="${child}Modifications" value="">
END_OF_SCREEN

            print <<END_OF_SCREEN;
<div class="row">
<fieldset id="${child}Fieldset">
<legend>${child}s</legend>
<div class="activationTable" id="${child}Activation">
<div class="headRow">
END_OF_SCREEN

            # Build a list of association fields
            my @fields   = ();
            my %dataType = ();
            my %default  = ();
            foreach my $name (
                sort {
                    $attributes{$association}{$a}
                      {Sequence} <=> $attributes{$association}{$b}{Sequence}
                } keys %{$attributes{$association}}
              )
            {
                my $hidden     = $attributes{$association}{$name}{Hidden};
                my $primaryKey = $attributes{$association}{$name}{PrimaryKey};
                my $required   = $attributes{$association}{$name}{Required};
                my $dataType   = $attributes{$association}{$name}{DataType};
                my $values     = $attributes{$association}{$name}{Values};
                my $defaultValue =
                  $attributes{$association}{$name}{DefaultValue};
                my $description = $attributes{$association}{$name}{Description};

                next if $hidden eq "True";
                next if $name   eq $object;    # We don't want the parent key
                push @fields, $name;

                $dataType{$name} = $dataType;
                $default{$name}  = $defaultValue;
                if ($primaryKey eq "True")
                {
                    print "<div class=\"primaryKeyActivation\">$name</div>\n";
                }
                else
                {
                    print "<div class=\"active\">$name</div>\n";
                }
            }
            print "</div>\n";

            # Obtain child primary key
            my $key = "";
            foreach my $name (
                sort {
                    $attributes{$child}{$a}{Sequence} <=> $attributes{$child}
                      {$b}{Sequence}
                } keys %{$attributes{$child}}
              )
            {
                my $hidden     = $attributes{$child}{$name}{Hidden};
                my $primaryKey = $attributes{$child}{$name}{PrimaryKey};

                next if $hidden eq "True";
                if ($primaryKey eq "True")
                {
                    $key ||= $name;
                }
            }

            # Build and issue the association Request
            # SELECT @fields FROM $association where $object=$value
            my $request = new MAM::Request(
                object => $association,
                action => "Query",
                %tokenArgs
            );
            foreach my $name (@fields)
            {
                if ($name eq $key) { $request->addSelection($name, "Sort"); }
                else               { $request->addSelection($name); }
            }
            my $keyValue = "";
            foreach my $condition (split /,/, $conditions)
            {
                my ($name, $value) = split /=/, $condition;
                $keyValue ||= $value if $primaryKeys{$name} eq "True";
            }
            $request->addCondition($object, $keyValue);
            my $response = $request->getResponse();
            my $status   = $response->getStatus();
            my $message  = $response->getMessage();
            my $data     = $response->getDataElement();
            my $count    = $response->getCount();
            my $doc      = XML::LibXML::Document->new();
            $doc->setDocumentElement($data);

            my %children = ();
            foreach my $row ($data->childNodes())
            {
                my $childValue = "";
                foreach my $col ($row->childNodes())
                {
                    my $name  = $col->nodeName();
                    my $value = $col->textContent();
                    if ($name eq $key)
                    {
                        $childValue ||= $value;
                        push @originals, "${child}${value}";
                    }
                    else
                    {
                        $children{$childValue}{$name} = $value;
                    }
                }
            }

            # Build and issue the child Request
            # SELECT @names FROM $child
            $request = new MAM::Request(
                object => $child,
                action => "Query",
                %tokenArgs
            );
            $request->addSelection($key, "Sort");
            $response = $request->getResponse();
            $status   = $response->getStatus();
            $message  = $response->getMessage();
            $data     = $response->getDataElement();
            $count    = $response->getCount();
            $doc      = XML::LibXML::Document->new();
            $doc->setDocumentElement($data);

            # On success, add member data to the select
            if ($status ne "Failure")
            {
                my @members = ();
                foreach my $row ($data->childNodes())
                {
                    my $name  = ($row->childNodes())[0]->nodeName();
                    my $value = ($row->childNodes())[0]->textContent();
                    foreach my $field (@fields)
                    {
                        # We assume the primary key will always be listed first
                        if ($field eq $key)
                        {
                            push @members, $value;
                            my $style = "";
                            if (exists $children{$value})
                            {
                                $style = " style=\"display:block;\"";
                            }
                            print <<END_OF_SCREEN;
<div ${style}id="${value}${child}ActivationRow" class="activationRow">
<div class="primaryKeyActivation">$value</div>
END_OF_SCREEN
                        }
                        else
                        {
                            print "<div class=\"active\">\n";
                            my $preValue = "";
                            if (   exists $children{$value}
                                && exists $children{$value}{$field})
                            {
                                $preValue = $children{$value}{$field};
                            }
                            else
                            {
                                $preValue = $default{$field};
                            }
                            if ($dataType{$field} eq "Boolean")
                            {
                                my $checked = "";
                                if ($preValue eq "True")
                                {
                                    $checked = " CHECKED";
                                }
                                print
                                  "<input onchange=\"markModification(\'$value\',\'$child\');\" id=\"${value}${child}${field}\" name=\"${value}${child}${field}\" value=\"True\" type=\"checkbox\"$checked/>\n";
                            }
                            else
                            {
                                print
                                  "<input onchange=\"markModification(\'$value\',\'$child\');\" type=\"text\" name=\"${value}${child}${field}\" id=\"${value}${child}${field}\" value=\"$preValue\" size=\"10\" maxlength=\"80\"/>\n";
                            }
                            print "</div>\n";
                        }
                    }
                    print "</div>\n";
                }
                print <<END_OF_SCREEN;
</div>
<div id="${child}MultiSelect" class="multiSelect">
<label for="$child">${child}s</label>
<select multiple="yes" size="10" id="$child" name="$child" onchange="changeActivationTable(\'$child\');">
END_OF_SCREEN

                foreach my $member (@members)
                {
                    my $selected = "";
                    if (grep($_ eq $member, keys %children))
                    {
                        $selected = " SELECTED";
                    }
                    print
                      "<option value=\"$member\"$selected>$member</option>\n";
                }
                print "</select>\n</div>\n";
            }
            foreach my $field (@fields)
            {
                print
                  "<input id=\"${object}${child}${field}\" name=\"${object}${child}${field}\" type=\"hidden\">\n";
            }
            print "</fieldset>\n</div>\n";
        }
    }

    # Handle Fund Constraints
    if ($object eq "Fund")
    {
        print <<END_OF_SCREEN;
<p id="selectInstructions">New constraints will not be considered for addition until after you press the Add button.</p>
END_OF_SCREEN

        my $child       = "Constraint";
        my $association = "Constraint";

        print <<END_OF_SCREEN;
  <input type="hidden" name="${child}Creations" id="${child}Creations" value="">
  <input type="hidden" name="${child}Deletions" id="${child}Deletions" value="">
  <input type="hidden" name="${child}Modifications" id="${child}Modifications" value="">
END_OF_SCREEN

        print <<END_OF_SCREEN;
<div class="row">
<fieldset id="${child}Fieldset">
<legend>${child}s</legend>
<div id="${child}Activation">
<div class="headRow">
END_OF_SCREEN

        # Print Delete checkbox
        print "<div class=\"active\">Delete?</div>\n";

        # Build a list of association fields
        my @fields   = ();
        my %dataType = ();
        my %default  = ();
        my %fixed    = ();
        foreach my $name (
            sort {
                $attributes{$association}{$a}
                  {Sequence} <=> $attributes{$association}{$b}{Sequence}
            } keys %{$attributes{$association}}
          )
        {
            my $hidden       = $attributes{$association}{$name}{Hidden};
            my $primaryKey   = $attributes{$association}{$name}{PrimaryKey};
            my $required     = $attributes{$association}{$name}{Required};
            my $fixed        = $attributes{$association}{$name}{Fixed};
            my $dataType     = $attributes{$association}{$name}{DataType};
            my $values       = $attributes{$association}{$name}{Values};
            my $defaultValue = $attributes{$association}{$name}{DefaultValue};
            my $description  = $attributes{$association}{$name}{Description};

            next if $hidden eq "True";
            next if $name   eq $object;    # We don't want the parent key
            push @fields, $name;

            $dataType{$name} = $dataType;
            $default{$name}  = $defaultValue;
            $fixed{$name}    = $fixed;
            print "<div class=\"active\">$name</div>\n";
        }
        print "</div>\n";

        # Obtain child primary key
        my $key = "Id";

        # Build and issue the association Request
        # SELECT @fields FROM $association where $object=$value
        my $request = new MAM::Request(
            object => $association,
            action => "Query",
            %tokenArgs
        );
        foreach my $name (@fields)
        {
            if ($name eq $key) { $request->addSelection($name, "Sort"); }
            else               { $request->addSelection($name); }
        }
        my $keyValue = "";
        foreach my $condition (split /,/, $conditions)
        {
            my ($name, $value) = split /=/, $condition;
            $keyValue ||= $value if $primaryKeys{$name} eq "True";
        }
        $request->addCondition($object, $keyValue);
        my $response = $request->getResponse();
        my $status   = $response->getStatus();
        my $message  = $response->getMessage();
        my $data     = $response->getDataElement();
        my $count    = $response->getCount();
        my $doc      = XML::LibXML::Document->new();
        $doc->setDocumentElement($data);

        my %children = ();
        foreach my $row ($data->childNodes())
        {
            my $childValue = "";
            foreach my $col ($row->childNodes())
            {
                my $name  = $col->nodeName();
                my $value = $col->textContent();
                if ($name eq $key)
                {
                    $childValue ||= $value;
                    push @originals, "${child}${value}";
                }
                else
                {
                    $children{$childValue}{$name} = $value;
                }
            }
        }

        foreach my $childValue (sort keys %children)
        {
            print <<END_OF_SCREEN;
<div style="display:block;" id="${childValue}${child}ActivationRow" class="activationRow">
<div class="active">
<input onchange="markDeletion('$childValue','$child');" id="${childValue}${child}Delete" name="${childValue}${child}Delete" value="True" type="checkbox"/>
</div>
END_OF_SCREEN
            foreach my $field (@fields)
            {
                my $value = $children{$childValue}{$field};
                # We assume the primary key will always be listed first
                if ($field eq $key)
                {
                    print "<div class=\"active\">$childValue</div>\n";
                }
                else
                {
                    print "<div class=\"active\">\n";
                    if ($fixed{$field} eq "True")
                    {
                        print
                          "<input disabled=\"y\" type=\"text\" name=\"${childValue}${child}${field}\" id=\"${childValue}${child}${field}\" value=\"$value\" size=\"10\" maxlength=\"80\"/>\n";
                    }
                    elsif ($dataType{$field} eq "Boolean")
                    {
                        my $checked = "";
                        if ($value eq "True")
                        {
                            $checked = " CHECKED";
                        }
                        print
                          "<input onchange=\"markModification(\'$childValue\',\'$child\');\" id=\"${childValue}${child}${field}\" name=\"${childValue}${child}${field}\" value=\"True\" type=\"checkbox\"$checked/>\n";
                    }
                    else
                    {
                        print
                          "<input onchange=\"markModification(\'$childValue\',\'$child\');\" class=\"${field}\" type=\"text\" name=\"${childValue}${child}${field}\" id=\"${childValue}${child}${field}\" value=\"$value\" size=\"10\" maxlength=\"80\"/>\n";
                    }
                    print "</div>\n";
                }
            }
            print "</div>\n";
        }

        # Print an add row
        print <<END_OF_SCREEN;
<div style="display:block;" id="Add${child}ActivationRow" class="activationRow">
<div class="active">
<input class="button" onClick="addCreation('$child');" id="Add${child}Row" name="Add${child}Row" value="Add" type="button"/>
</div>
END_OF_SCREEN
        foreach my $field (@fields)
        {
            # We assume the primary key will always be listed first
            if ($field eq $key)
            {
                print "<div class=\"active\"></div>\n";
            }
            else
            {
                print "<div class=\"active\">\n";
                if ($dataType{$field} eq "Boolean")
                {
                    my $checked = "";
                    if ($default{$field} eq "True")
                    {
                        $checked = " CHECKED";
                    }
                    print
                      "<input id=\"Add${child}${field}\" name=\"${field}\" value=\"True\" type=\"checkbox\"$checked/>\n";
                }
                else
                {
                    print
                      "<input class=\"${field}\" type=\"text\" name=\"${field}\" id=\"Add${child}${field}\" value=\"$default{$field}\" size=\"10\" maxlength=\"80\"/>\n";
                }
                print "</div>\n";
            }
        }
        print "</div>\n";

        foreach my $field (@fields)
        {
            print
              "<input id=\"${object}${child}${field}\" name=\"${object}${child}${field}\" type=\"hidden\">\n";
        }
        print "</fieldset>\n</div>\n";
    }

    print <<END_OF_SCREEN_BOTTOM;
  <div id="submitRow">
    <input class="button" type="submit" value="Modify $object">
  </div>
</form>
END_OF_SCREEN_BOTTOM

    # Build original array of association values in javascript
    my $originalArray = join ',', map "\"$_\"", @originals;
    print <<END_OF_JAVASCRIPT;
<script language="Javascript">
var originalArray = new Array($originalArray);
</script>
END_OF_JAVASCRIPT
}

