#! /usr/bin/perl -wT
################################################################################
#
# Reset Form Frame for Web GUI
#
# File   :  prereset.cgi
#
################################################################################
#
#    This file includes parts which are copyrighted by Adaptive Computing
#    Enterprises, Inc., and parts which are copyrighted by Battelle Memorial
#    Institute.
#
#    The terms and conditions for the use and redistribution of these parts
#    are governed by the Moab Accounting Manager License and the BSD License
#    respectively. Refer to the LICENSE file for details.
#
#    Copyright (C) 2006 - 2016 by Adaptive Computing Enterprises, Inc.
#    All Rights Reserved.
#
#    The Moab Accounting Manager License specifies the terms and conditions
#    for use and redistribution.
#
#    The Moab Accounting Manager License applies to the Moab Accounting
#    Manager software offered by Adaptive Computing Enterprises, Inc.
#    By installing or using this software, Licensee accepts a non-exclusive
#    license from Adaptive Computing Enterprises, Inc. and is bound to accept
#    acknowledgement of and abide by the notices and conditions of the
#    Moab Accounting Manager License.
#
#    Copyright (C) 2003 - 2005 Pacific Northwest National Laboratory,
#    Battelle Memorial Institute. All rights reserved.
#
#    The BSD License specifies the terms and conditions for use and
#    redistribution.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#
#    - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
#    - Neither the name of Battelle nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#    POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

use strict;
use lib qw(/opt/mam/lib /opt/mam/lib/perl5);
use CGI qw(:standard);
use CGI::Session;
use XML::LibXML;
use MAM::CGI;
use MAM::Global;

my $cgi        = new CGI;
my $session    = new CGI::Session(undef, $cgi, {Directory => "/tmp"});
my $username   = $session->param("username");
my $password   = $session->param("password");
my $remoteUser = remote_user();
unless (($username && $password) || defined $remoteUser)
{
    print header;
    print start_html(
        -title  => "Session Expired",
        -onLoad => "top.location.replace(\"login.cgi\");"
    );
    print end_html;
    exit(0);
}

my $object    = $cgi->param("myobject");
my $action    = $cgi->param("myaction");

# Javascript to changeActivationTable and getValue
my $java_script = <<END_JAVA_SCRIPT;

function newWindow(windowName, url, winWidth, maxHeight) {
    var winHeight = ((screen.availHeight-100) > maxHeight) ? maxHeight : screen.availHeight-100;
                                                                                
    var nw=window.open(url, windowName, "scrollbars,status,resizable,width=" + winWidth + ",height=" + winHeight);
    if(!nw) return null;
                                                                                
    if(parseInt(navigator.appVersion) >= 4) {
        var nwLeft = parseInt((screen.availWidth - winWidth) / 2);
        var nwTop = parseInt((screen.availHeight - winHeight) / 2);                                                                                 
        if(parseInt(navigator.appVersion) >= 4) {
            nw.moveTo(nwLeft, nwTop-50);
        }
    }
    nw.focus();
    return nw;
}

function setId(id){
    document.getElementById('Id').value = id;
}
        
function selectFund(){
    newWindow('Select_Fund', 'selectFund.cgi', 750, 500);
}
END_JAVA_SCRIPT

# Print the reset form
print header;
print start_html(
    -title  => "Reset Funds",
    -script => {-code => $java_script},
    -style  => {-src => "/cgi-bin/mam/styles/filter.css"},
);
print_body();
print end_html;

sub print_body
{
    # Print the header
    my @headerParts = ({-class => "header"});
    push @headerParts, h1("Reset Accounts");
    print div(@headerParts);

    # Print text boxes
    print <<END_OF_HTML;
<p xmlns="" class="required" id="requiredDesc">Warning -- If no fund is specified, ALL funds will be reset.</p>
<form xmlns="" name="inputForm" method="post" action="reset.cgi">
  <input type="hidden" name="myobject" value="$object">
  <input type="hidden" name="myaction" value="$action">
  <div class="row">
    <label for="Id" class="rowLabel">Fund Id:</label>
    <input type="text" name="Id" id="Id" value="" size="26" maxlength=""/>
    <input class="button" type="button" name="selectFundButton" id="selectFundButton" value="Select..." onClick="selectFund()"/>
  </div>
END_OF_HTML

    print <<END_OF_HTML;
  <div id="submitRow">
    <input type="submit" value="Reset Funds"/>
  </div>
</form>
END_OF_HTML
}

