#! /usr/bin/perl -wT
################################################################################
#
# Prescreen Condition Frame for Web GUI
#
# File   :  prescreen.cgi
#
################################################################################
#
#    This file includes parts which are copyrighted by Adaptive Computing
#    Enterprises, Inc., and parts which are copyrighted by Battelle Memorial
#    Institute.
#
#    The terms and conditions for the use and redistribution of these parts
#    are governed by the Moab Accounting Manager License and the BSD License
#    respectively. Refer to the LICENSE file for details.
#
#    Copyright (C) 2006 - 2016 by Adaptive Computing Enterprises, Inc.
#    All Rights Reserved.
#
#    The Moab Accounting Manager License specifies the terms and conditions
#    for use and redistribution.
#
#    The Moab Accounting Manager License applies to the Moab Accounting
#    Manager software offered by Adaptive Computing Enterprises, Inc.
#    By installing or using this software, Licensee accepts a non-exclusive
#    license from Adaptive Computing Enterprises, Inc. and is bound to accept
#    acknowledgement of and abide by the notices and conditions of the
#    Moab Accounting Manager License.
#
#    Copyright (C) 2003 - 2005 Pacific Northwest National Laboratory,
#    Battelle Memorial Institute. All rights reserved.
#
#    The BSD License specifies the terms and conditions for use and
#    redistribution.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#
#    - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
#    - Neither the name of Battelle nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#    POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

use strict;
use lib qw(/opt/mam/lib /opt/mam/lib/perl5);
use CGI qw(:standard);
use CGI::Session;
use XML::LibXML;
use MAM::CGI;
use MAM::Global;

my $cgi        = new CGI;
my $session    = new CGI::Session(undef, $cgi, {Directory => "/tmp"});
my $username   = $session->param("username");
my $password   = $session->param("password");
my $remoteUser = remote_user();
unless (($username && $password) || defined $remoteUser)
{
    print header;
    print start_html(
        -title  => "Session Expired",
        -onLoad => "top.location.replace(\"login.cgi\");"
    );
    print end_html;
    exit(0);
}
my %tokenArgs =
  defined $remoteUser
  ? (actor => $remoteUser, tokenType => $TOKEN_SYMMETRIC)
  : (
    actor      => "$username",
    tokenType  => $TOKEN_PASSWORD,
    tokenName  => "$username",
    tokenValue => "$password"
  );

my $objects_ref          = $session->param("objects");
my %objects              = %{$objects_ref};
my $actions_ref          = $session->param("actions");
my %actions              = %{$actions_ref};
my $attributes_ref       = $session->param("attributes");
my %attributes           = %{$attributes_ref};
my $object               = $cgi->param("myobject");
my $action               = $cgi->param("myaction");
my $currency_enablehours = $clientConfig->{'currency.enablehours'};
my %discriminators       = ();
my %constraints          = ();

# Javascript for changeOption and changeCondition
my $java_script = <<END_JAVA_SCRIPT;
function changeCondition(component,name){
  var conditionNamesInput = document.getElementById("conditionNames");
  var conditionValuesInput = document.getElementById("conditionValues");
  //alert("before names=" + conditionNamesInput.value);
  //alert("before values=" + conditionValuesInput.value);
  var componentValue = component.value;
  
  var nameList = conditionNamesInput.value.split(",");
  var valueList = conditionValuesInput.value.split(",");

  if (nameList == "") {
    nameList = [];
    valueList = [];
  }

  // Take the name out of the list
  for (var i in nameList) {
    if (nameList[i] == name) {
     nameList.splice(i, 1);
     valueList.splice(i, 1);
    }
  }

  // If we have a value, append it back on to the list
  if (componentValue != "")
  {
    nameList.push(name);
    valueList.push(componentValue);
  }

  conditionNamesInput.value = nameList.join(',');
  conditionValuesInput.value = valueList.join(',');
  //alert("after names=" + conditionNamesInput.value);
  //alert("after values=" + conditionValuesInput.value);
}

function changeOption(component,name){
  var optionNamesInput = document.getElementById("optionNames");
  var optionValuesInput = document.getElementById("optionValues");
  // alert("before names=" + hiddenNamesInput.value);
  // alert("before values=" + hiddenValuesInput.value);
  var componentValue = component.value;
  
  var nameList = optionNamesInput.value.split(",");
  var valueList = optionValuesInput.value.split(",");

  if (nameList == "") {
    nameList = [];
    valueList = [];
  }

  // Take the name out of the list
  for (var i in nameList) {
    if (nameList[i] == name) {
     nameList.splice(i, 1);
     valueList.splice(i, 1);
    }
  }

  // If we have a value, append it back on to the list
  if (componentValue != "")
  {
    nameList.push(name);
    valueList.push(componentValue);
  }

  optionNamesInput.value = nameList.join(',');
  optionValuesInput.value = valueList.join(',');
  // alert("after names=" + hiddenNamesInput.value);
}

function changeFilter(component,name){
  var filterNamesInput = document.getElementById("filterNames");
  var filterValuesInput = document.getElementById("filterValues");
  // alert("before names=" + optionNamesInput.value);
  // alert("before values=" + optionValuesInput.value);
  var componentValue = component.value;
  
  var nameList = filterNamesInput.value.split(",");
  var valueList = filterValuesInput.value.split(",");

  if (nameList == "") {
    nameList = [];
    valueList = [];
  }

  // Take the name out of the list
  for (var i in nameList) {
    if (nameList[i] == name) {
     nameList.splice(i, 1);
     valueList.splice(i, 1);
    }
  }

  // If we have a value, append it back on to the list
  if (componentValue != "")
  {
    nameList.push(name);
    valueList.push(componentValue);
  }

  filterNamesInput.value = nameList.join(',');
  filterValuesInput.value = valueList.join(',');
  // alert("after names=" + hiddenNamesInput.value);
}
END_JAVA_SCRIPT

print header;
print start_html(
    -title  => "Prefilter ${object}s",
    -script => [{-code => $java_script}],
    -style  => {-src => "/cgi-bin/mam/styles/filter.css"}
);
print_body();
print end_html;

sub print_body
{

    # Print the header
    my @headerParts = ({-class => "header"});
    push @headerParts, h1("Prefilter ${object}s");
    print div(@headerParts);

    # Print the prescreen form
    print <<END_OF_SCREEN_TOP;
<form name="screenForm" method="post" action="list.cgi">
  <input type="hidden" name="myobject" value="$object">
  <input type="hidden" name="myaction" value="$action">
  <input type="hidden" name="conditionNames" id="conditionNames" value="">
  <input type="hidden" name="conditionValues" id="conditionValues" value="">
  <input type="hidden" name="optionNames" id="optionNames" value="">
  <input type="hidden" name="optionValues" id="optionValues" value="">
  <input type="hidden" name="filterNames" id="filterNames" value="">
  <input type="hidden" name="filterValues" id="filterValues" value="">
END_OF_SCREEN_TOP

    # Object and Action
    if ($object eq "Transaction")
    {
        my $name = "Object";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <div id="${name}MultiSelect" class="multiSelect">
      <label class="rowLabel" for="$name">${name}:</label>
      <select onchange="changeCondition(this,'$name');" id="$name" name="$name">
        <option value="">ANY $name</option>
END_OF_SCREEN_SECTION
        foreach my $obj (sort keys %objects)
        {
            print "        <option value=\"$obj\">$obj</option>\n";
        }
        print "      </select>\n    </div>\n  </div>\n";

        $name = "Action";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <div id="${name}MultiSelect" class="multiSelect">
      <label class="rowLabel" for="$name">${name}:</label>
      <select onchange="changeCondition(this,'$name');" id="$name" name="$name">
        <option value="">ANY $name</option>
END_OF_SCREEN_SECTION
        my %uniqueActions = ();
        foreach my $obj (keys %actions)
        {
            foreach my $act (keys %{$actions{$obj}})
            {
                $uniqueActions{$act}++;
            }
        }
        foreach my $act (sort keys %uniqueActions)
        {
            print "        <option value=\"$act\">$act</option>\n";
        }
        print "      </select>\n    </div>\n  </div>\n";
    }

    # Id
    if (   $object eq "Fund"
        || $object eq "Allocation"
        || $object eq "Payment"
        || $object eq "Quote"
        || $object eq "Lien"
        || $object eq "UsageRecord")
    {
        my $name = "Id";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">$object Id:</label>
    <input onchange="changeCondition(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION
    }

    # UsageRecord (Charge)
    if ($object eq "Charge")
    {
        my $name = "UsageRecord";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">Usage Record:</label>
    <input onchange="changeCondition(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION
    }

    # Instance
    if (   $object eq "Charge"
        || $object eq "Payment"
        || $object eq "Quote"
        || $object eq "Lien"
        || $object eq "Transaction"
        || $object eq "UsageRecord")
    {
        my $name = "Instance";
        my $construct = $object eq "Payment" ? "Filter" : "Condition";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">Instance Pattern:</label>
    <input onchange="change$construct(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION
    }

    # Name
    if ($object eq "Fund" || $object eq "Charge")
    {
        my $name = "Name";
        my $label =
          $object eq "Fund"
          ? "Fund Name Pattern"
          : "Usage Record Property Name";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">${label}:</label>
    <input onchange="changeCondition(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION
    }

    # Fund
    if ($object eq "Allocation")
    {
        my $name = "Fund";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">Fund Id:</label>
    <input onchange="changeCondition(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION
    }

    # Active
    if (   $object eq "Allocation"
        || $object eq "Payment"
        || $object eq "Quote"
        || $object eq "Lien")
    {
        my $construct = $object eq "Allocation" ? "Condition" : "Option";
        my $name = "Active";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="name(./*[1])">${name}:</label>
    <label for="ActiveYes">Yes</label>
    <input onchange="change${construct}(this,'$name');" type="radio" name="$name" id="ActiveYes" value="True">
    <label for="ActiveNo">No</label>
    <input onchange="change${construct}(this,'$name');" type="radio" name="$name" id="ActiveNo" value="False">
    <label for="ActiveANY">ANY</label>
    <input onchange="change${construct}(this,'$name');" type="radio" name="$name" id="ActiveANY" value="" checked="true">
  </div>
END_OF_SCREEN_SECTION
    }

    # StartTime and EndTime
    if (   $object eq "Payment"
        || $object eq "Transaction"
        || $object eq "UsageRecord")
    {
        my $name = $object eq "Payment" ? "BeginTime" : "StartTime";
        my $word = $object eq "Payment" ? "Begin"     : "Start";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">Earliest $word Time (YYYY-MM-DD):</label>
    <input onchange="changeCondition(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION

        $name = "EndTime";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">Latest End Time (YYYY-MM-DD):</label>
    <input onchange="changeCondition(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION
    }

    # CreationTime
    if ($object eq "Charge")
    {
        my $name = "EarliestCreationTime";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">Earliest $object Time (YYYY-MM-DD):</label>
    <input onchange="changeCondition(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION

        $name = "LatestCreationTime";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">Latest $object Time (YYYY-MM-DD):</label>
    <input onchange="changeCondition(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION
    }

    # Type and Stage
    if ($object eq "UsageRecord")
    {
        my $name = "Type";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">Usage Type:</label>
    <input onchange="changeCondition(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION

        $name = "Stage";
        my $values       = $attributes{"UsageRecord"}{"Stage"}{"Values"};
        my $defaultValue = $attributes{"UsageRecord"}{"Stage"}{"DefaultValue"};
        if ($values =~ /^\((.*)\)$/)
        {
            print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">Usage Stage:</label>
    <select onchange="changeCondition(this,'$name');" id="$name" name="$name">
END_OF_SCREEN_SECTION

            print
              "      <Option value=\"$defaultValue\">$defaultValue</Option>\n";
            foreach my $value (split /,/, $1)
            {
                print "      <Option value=\"$value\">$value</Option>\n";
            }
            print "    </select>\n";
            print "</div>\n";
        }
    }

    # Fund and UsageRecord (Transaction)
    if ($object eq "Transaction")
    {
        my $name = "Fund";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">Fund Id:</label>
    <input onchange="changeCondition(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION

        $name = "UsageRecord";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">Usage Record:</label>
    <input onchange="changeCondition(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION
    }

    # Transaction Discriminators
    if ($object eq "Transaction")
    {
        # Identify discriminators by finding the simple difference
        # between the regular Transaction attributes and the actual list
        my %nonDiscriminators = ();
        @nonDiscriminators{
            "Id",           "Object",           "Action",
            "Actor",        "Key",              "Child",
            "Count",        "Details",          "Description",
            "CreationTime", "ModificationTime", "Deleted",
            "RequestId",    "TransactionId",    "Instance",
            "Amount",       "Delta",            "Balance",
            "Remaining",    "Fund",             "Allocation",
            "UsageRecord",  "Duration",
          }
          = ();
        foreach my $name (keys %{$attributes{"Transaction"}})
        {
            $discriminators{$name} = '' unless exists $nonDiscriminators{$name};
        }

        foreach my $name (sort keys %discriminators)
        {
            print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">${name}:</label>
    <input onchange="changeCondition(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION
        }
    }

    # Usage Record Discriminators
    if (   $object eq "Payment"
        || $object eq "Quote"
        || $object eq "Lien"
        || $object eq "UsageRecord")
    {
        my $construct = $object eq "UsageRecord" ? "Condition" : "Filter";

        # Identify discriminators by finding the simple difference
        # between the regular UsageRecord attributes and the actual list
        my %nonDiscriminators = ();
        @nonDiscriminators{
            "Id",                "Type",        "Instance",
            "Charge",            "Stage",       "Quote",
            "Nodes",             "Processors",  "CPUTime",
            "Memory",            "Disk",        "Duration",
            "RequestedDuration", "SubmitTime",  "StartTime",
            "EndTime",           "Description", "CreationTime",
            "ModificationTime",  "Deleted",     "RequestId",
            "TransactionId"
          }
          = ();
        foreach my $name (keys %{$attributes{"UsageRecord"}})
        {
            $discriminators{$name} = '' unless exists $nonDiscriminators{$name};
        }

        foreach my $name (sort keys %discriminators)
        {
            print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">${name}:</label>
    <input onchange="change${construct}(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION
        }
    }

    # Fund Constraints
    if (   $object eq "Fund"
        || $object eq "Allocation"
        || $object eq "Lien")
    {
        # Constraint Query
        my $request = new MAM::Request(
            object => "Constraint",
            action => "Query",
            %tokenArgs
        );
        $request->addSelection("Name");
        $request->addSelection("Value");
        my $response = $request->getResponse();
        my $status   = $response->getStatus();
        my $message  = $response->getMessage();
        my $data     = $response->getDataElement();
        my $count    = $response->getCount();
        my $doc      = XML::LibXML::Document->new();
        $doc->setDocumentElement($data);

        # Build up list of constraints
        foreach my $row ($data->childNodes())
        {
            my $name  = ($row->childNodes())[0]->textContent();
            my $value = ($row->childNodes())[1]->textContent();
            $constraints{$name}{$value}++ unless $value =~ /^!/;
        }

        foreach my $name (sort keys %constraints)
        {
            next if defined $discriminators{$name};

            print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="$name">${name}:</label>
    <input onchange="changeFilter(this,'$name');" maxlength="" size="26" value="" id="$name" name="$name" type="text"/>
  </div>
END_OF_SCREEN_SECTION
        }
    }

    # Exclusivity Filter
    if ($object eq "Fund" || $object eq "Allocation")
    {
        my $name = "FilterType";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="name(./*[1])">${name}:</label>
    <label for="FilterTypeExactMatch">ExactMatch</label>
    <input onchange="changeOption(this,'$name');" type="radio" name="$name" id="FilterTypeExactMatch" value="ExactMatch">
    <label for="FilterTypeExclusive">Exclusive</label>
    <input onchange="changeOption(this,'$name');" type="radio" name="$name" id="FilterTypeExclusive" value="Exclusive">
    <label for="FilterTypeNonExclusive">NonExclusive</label>
    <input onchange="changeOption(this,'$name');" type="radio" name="$name" id="FilterTypeNonExclusive" value="NonExclusive" checked="true">
  </div>
END_OF_SCREEN_SECTION
    }

    # Lien Filter
    if ($object eq "Lien")
    {
        my $name = "FilterType";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="name(./*[1])">${name}:</label>
    <label for="FilterTypeAttributedTo">AttributedTo</label>
    <input onchange="changeOption(this,'$name');" type="radio" name="$name" id="FilterTypeAttributedTo" value="AttributedTo" checked="true">
    <label for="FilterTypeImpingesUpon">ImpingesUpon</label>
    <input onchange="changeOption(this,'$name');" type="radio" name="$name" id="FilterImpingesUpon" value="ImpingesUpon">
  </div>
END_OF_SCREEN_SECTION
    }

    # Show Hours
    if (   $object eq "Fund"
        && defined $currency_enablehours
        && $currency_enablehours =~ /true/i)
    {
        my $name = "ShowHours";
        print <<END_OF_SCREEN_SECTION;
  <div class="row">
    <label class="rowLabel" for="name(./*[1])">${name}:</label>
    <label for="ShowHoursYes">True</label>
    <input onchange="changeOption(this,'$name');" type="radio" name="$name" id="ShowHoursYes" value="True">
    <label for="ShowHoursNo">False</label>
    <input onchange="changeOption(this,'$name');" type="radio" name="$name" id="ShowHoursNo" value="False" checked="true">
  </div>
END_OF_SCREEN_SECTION
    }

    print <<END_OF_SCREEN_BOTTOM;
  <div id="submitRow">
    <input class="button" type="submit" value="Prefilter ${object}s">
  </div>
</form>
END_OF_SCREEN_BOTTOM
}
