#! /usr/bin/perl -wT
################################################################################
#
# Statement Frame for Web GUI
#
# File   :  statement.cgi
#
################################################################################
#
#    This file includes parts which are copyrighted by Adaptive Computing
#    Enterprises, Inc., and parts which are copyrighted by Battelle Memorial
#    Institute.
#
#    The terms and conditions for the use and redistribution of these parts
#    are governed by the Moab Accounting Manager License and the BSD License
#    respectively. Refer to the LICENSE file for details.
#
#    Copyright (C) 2006 - 2016 by Adaptive Computing Enterprises, Inc.
#    All Rights Reserved.
#
#    The Moab Accounting Manager License specifies the terms and conditions
#    for use and redistribution.
#
#    The Moab Accounting Manager License applies to the Moab Accounting
#    Manager software offered by Adaptive Computing Enterprises, Inc.
#    By installing or using this software, Licensee accepts a non-exclusive
#    license from Adaptive Computing Enterprises, Inc. and is bound to accept
#    acknowledgement of and abide by the notices and conditions of the
#    Moab Accounting Manager License.
#
#    Copyright (C) 2003 - 2005 Pacific Northwest National Laboratory,
#    Battelle Memorial Institute. All rights reserved.
#
#    The BSD License specifies the terms and conditions for use and
#    redistribution.
#
#    Redistribution and use in source and binary forms, with or without
#    modification, are permitted provided that the following conditions
#    are met:
#
#    - Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    - Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
#    - Neither the name of Battelle nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
#    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
#    FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
#    COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
#    INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#    BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
#    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#    LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
#    ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#    POSSIBILITY OF SUCH DAMAGE.
#
################################################################################

use strict;
use lib qw(/opt/mam/lib /opt/mam/lib/perl5);
use CGI qw(:standard);
use CGI::Session;
use XML::LibXML;
use MAM::CGI;
use MAM::Global;

my $cgi        = new CGI;
my $session    = new CGI::Session(undef, $cgi, {Directory => "/tmp"});
my $username   = $session->param("username");
my $password   = $session->param("password");
my $remoteUser = remote_user;
unless (($username && $password) || defined $remoteUser)
{
    print header;
    print start_html(
        -title  => "Session Expired",
        -onLoad => "top.location.replace(\"login.cgi\");"
    );
    print end_html;
    exit(0);
}
my %tokenArgs =
  defined $remoteUser
  ? (actor => $remoteUser, tokenType => $TOKEN_SYMMETRIC)
  : (
    actor      => "$username",
    tokenType  => $TOKEN_PASSWORD,
    tokenName  => "$username",
    tokenValue => "$password"
  );

my $object               = $cgi->param("myobject");
my $action               = $cgi->param("myaction");
my $fund                 = $cgi->param("Fund");
my $constraintNames      = $cgi->param("constraintNames") || '';
my $constraintValues     = $cgi->param("constraintValues") || '';
my $startTime            = $cgi->param("StartTime");
my $endTime              = $cgi->param("EndTime");
my $itemize              = $cgi->param("Itemize");
my $filterType           = $cgi->param("FilterType");
my $currency_enablehours = $clientConfig->{'currency.enablehours'};
my $showHours =
  $cgi->param("ShowHours")
  || (defined $currency_enablehours && $currency_enablehours =~ /true/i)
  ? 'True'
  : 'False';
my $display = $cgi->param("display");
my $time_division = $showHours eq "True" ? 3600 : 1;
my $currency_precision =
  $showHours eq "True" ? 2 : $clientConfig->{'currency.precision'} || 0;

# Derive discriminators
my $defaultDiscriminators = 'Account,User,Machine';
my $discriminators = $clientConfig->{'statement.discriminators'}
  || $defaultDiscriminators;
if ($discriminators =~ /^([\w_,]+)$/) { $discriminators = $1; }    # Untaint
else
{
    die "Illegal characters were found in \$discriminators ($discriminators)\n";
}
my @discriminators = split ',', $discriminators;

# Untaint currency precision
if ($currency_precision =~ /^(\d+)$/)
{
    $currency_precision = $1;
}
else
{
    die
      "Illegal characters were found in \$currency_precision ($currency_precision)\n";
}

# Parse out selected constraints
my %selected = ();
@selected{split /,/, $constraintNames} = (split /,/, $constraintValues);

# Javascript to changeActivationTable and getValue
my $java_script = <<END_JAVA_SCRIPT;
function newWindow(windowName, url, winWidth, maxHeight) {
    var winHeight = ((screen.availHeight-100) > maxHeight) ? maxHeight : screen.availHeight-100;
                                                                                
    var nw=window.open(url, windowName, "scrollbars,status,resizable,width=" + winWidth + ",height=" + winHeight);
    if(!nw) return null;
                                                                                
    if(parseInt(navigator.appVersion) >= 4) {
        var nwLeft = parseInt((screen.availWidth - winWidth) / 2);
        var nwTop = parseInt((screen.availHeight - winHeight) / 2);                                                                                 
        if(parseInt(navigator.appVersion) >= 4) {
            nw.moveTo(nwLeft, nwTop-50);
        }
    }
    nw.focus();
    return nw;
}
                                                                                
function setId(id){
  document.getElementById('Fund').value = id;
}
                                                                                
function selectFund(){
  newWindow('Select.Fund', 'selectFund.cgi', 750, 500);
}
                                                                                
function updateStatus(statusInfo) {
  if (statusInfo) {
    parent.mamstatusbar.document.location="status.cgi?" + statusInfo;
    document.resultsForm.submit();
  }
}

function markChange(name){
  var hiddenNamesInput = document.getElementById("constraintNames");
  var hiddenValuesInput = document.getElementById("constraintValues");
  // alert("before names=" + hiddenNamesInput.value);
  // alert("before values=" + hiddenValuesInput.value);
  var mySelectValue = document.getElementById(name).value;
  
  var nameList = hiddenNamesInput.value.split(",");
  var valueList = hiddenValuesInput.value.split(",");
  if (nameList == "") {
    nameList = [];
    valueList = [];
  }
  for (var i in nameList) {
    if (nameList[i] == name) {
     nameList.splice(i, 1);
     valueList.splice(i, 1);
    }
  }

  // If we have a value, append it back on to the list
  if (mySelectValue != "")
  {
    nameList.push(name);
    valueList.push(mySelectValue);
  }
  hiddenNamesInput.value = nameList.join(',');
  hiddenValuesInput.value = valueList.join(',');
  // alert("after names=" + hiddenNamesInput.value);
  // alert("after values=" + hiddenValuesInput.value);
}
END_JAVA_SCRIPT

print header;
print start_html(
    -title  => "Fund Statement",
    -script => {-code => $java_script},
    -style  => {-src => "/cgi-bin/mam/styles/statement.css"}
);
print_body();
print end_html;

sub print_body
{
    # Print the header
    my @headerParts = ({-class => "header"});
    push @headerParts, h1("Fund Statement");
    print div(@headerParts);

    # Print the statement form
    print <<END_OF_HTML;
<form name="inputForm" method="post" action="statement.cgi">
  <input type="hidden" name="myobject" value="$object">
  <input type="hidden" name="myaction" value="$action">
  <input type="hidden" name="constraintNames" id="constraintNames" value="">
  <input type="hidden" name="constraintValues" id="constraintValues" value="">
  <input type="hidden" name="display" value="True">
  <div class="row">
    <label for="Fund" class="rowLabel">Fund Id:</label>
    <input type="text" name="Fund" id="Fund" value="" size="26" maxlength=""/>
    <input class="button" type="button" name="selectFundButton" id="selectFundButton" value="Select..." onClick="selectFund()"/>
  </div>
END_OF_HTML

    # Build up list of constraints
    my %constraints = ();

    # Constraint Query
    my $request = new MAM::Request(
        object => "Constraint",
        action => "Query",
        %tokenArgs
    );
    $request->addSelection("Name");
    $request->addSelection("Value");
    my $response = $request->getResponse();
    my $status   = $response->getStatus();
    my $message  = $response->getMessage();
    my $data     = $response->getDataElement();
    my $count    = $response->getCount();
    my $doc      = XML::LibXML::Document->new();
    $doc->setDocumentElement($data);

    foreach my $row ($data->childNodes())
    {
        my $name  = ($row->childNodes())[0]->textContent();
        my $value = ($row->childNodes())[1]->textContent();
        $constraints{$name}{$value}++ unless $value =~ /^!/;
    }

    foreach my $name (sort keys %constraints)
    {
        # Print Constraint Select
        print <<END_OF_HTML;
    <div class="row">
      <label class="rowLabel" for="$name">${name}:</label>
      <input type="text" onchange="markChange('$name');" name="$name" id="$name" value="" size="26" maxlength="80"/>
    </div>
END_OF_HTML
    }

    print <<END_OF_HTML;
  <div class="row">
    <label for="StartTime" class="rowLabel">Start Date:</label>
    <input type="text" name="StartTime" id="StartTime" value="" size="26" maxlength=""/>
  </div>
  <div class="row">
    <label for="EndTime" class="rowLabel">End Date:</label>
    <input type="text" name="EndTime" id="EndTime" value="" size="26" maxlength=""/>
  </div>
  <div class="row">
    <strong class="rowLabel">Itemize?</strong><input type="radio" name="Itemize" id="ItemizeYes" value="True"><label for="ItemizeYes">Yes</label><input type="radio" name="Itemize" id="ItemizeNo" value="False" checked="true"><label for="ItemizeNo">No</label>
  </div>
  <div class="row">
    <strong class="rowLabel">Filter Type?</strong><input type="radio" name="FilterType" id="FilterType" value="ExactMatch"><label for="FilterTypeExactMatch">ExactMatch</label><input type="radio" name="FilterType" id="FilterType" value="Exclusive"><label for="FilterTypeExclusive">Exclusive</label><input type="radio" name="FilterType" id="FilterTypeNonExclusive" value="NonExclusive" checked="true"><label for="FilterTypeNonExclusive">NonExclusive</label>
  </div>
END_OF_HTML

    if (defined $currency_enablehours && $currency_enablehours =~ /true/i)
    {
        print <<END_OF_HTML;
  <div class="row">
    <strong class="rowLabel">Show Hours?</strong><input type="radio" name="ShowHours" id="ShowHoursYes" value="True"><label for="ShowHoursYes">Yes</label><input type="radio" name="ShowHours" id="ShowHoursNo" value="False" checked="true"><label for="ShowHoursNo">No</label>
  </div>
END_OF_HTML
    }

    print <<END_OF_HTML;
  <div id="submitRow">
    <input class="button" value="Display Statement" type="submit"/>
  </div>
</form>
END_OF_HTML

    if ($display)
    {
        my %funds = ();

        unless ($startTime) { $startTime = "-Infinity"; }
        unless ($endTime)   { $endTime   = "Now"; }

        # Obtain list of applicable funds
        {
         # Fund Query Show:="Sort(Id),Name" [Id:=$fund] [Filter:=$filter]*
            my $request = new MAM::Request(
                object => "Fund",
                action => "Query",
                %tokenArgs
            );
            $request->addSelection("Id", "Sort");
            $request->addSelection("Name");
            $request->addCondition("Id", $fund) if $fund;
            foreach my $name (keys %selected)
            {
                $request->addOption("Filter", "${name}=$selected{$name}");
            }
            $request->setOption("FilterType", $filterType);
            my $response = $request->getResponse();
            my $status   = $response->getStatus();
            my $message  = $response->getMessage();
            my @data     = $response->getData();

            foreach my $datum (@data)
            {
                $funds{$datum->getPropertyValue("Id")} = $datum->getPropertyValue("Name");
            }
            if (scalar keys %funds == 0)
            {
                # Display an error message
                print end_html;
                print start_html(
                    -title  => "$PRODUCT Error",
                    -script => {-code => $java_script},
                    -onLoad =>
                      "updateStatus(\"message=No funds were matched. A fund statement cannot be produced for the chosen selections.\")"
                );
                return;
            }
        }

        print "<div id=\"available\">\n";
        print
          "<strong>Statement for fund $fund ($funds{$fund})</strong><br>\n"
          if $fund;
        foreach my $name (keys %selected)
        {
            print "<strong>Statement for $name $selected{$name}</strong><br>\n";
        }
        if (! $fund)
        {

            foreach my $id (sort keys %funds)
            {
                print
                  "<strong>Includes fund $id ($funds{$id})</strong><br>\n";
            }
        }
        print
          "<strong>Reporting fund activity from $startTime to $endTime</strong><br>\n";
        print "<br>\n";

        # Add up the beginning active allocations
        my $beginningBalance = 0;
        {
            my $request = new MAM::Request(
                object => "Allocation",
                action => "Query",
                %tokenArgs
            );
            setFunds($request, \%funds);
            $request->addCondition("Active", "True");
            $request->setOption("Time", $startTime);
            $request->addSelection("Amount", "Sum");
            my $response = $request->getResponse();

            if ($response->getStatus() eq "Failure")
            {
                my $code    = $response->getCode();
                my $message = $response->getMessage();
                print start_html(
                    -title  => "$PRODUCT Error",
                    -script => {-code => $java_script},
                    -onLoad =>
                      "updateStatus(\"message=Aborting Fund Statement: $message.\")"
                );
                return;
            }
            my $doc  = XML::LibXML::Document->new();
            my $data = $response->getDataElement();
            $doc->setDocumentElement($data);
            foreach my $row ($data->childNodes())
            {
                my $sum =
                  ($row->getChildrenByTagName("Amount"))[0]->textContent();
                $beginningBalance += $sum if $sum;
            }
        }

        # Print the beginning balance
        my $displayBeginningBalance = sprintf("%20.${currency_precision}f",
            $beginningBalance / $time_division);
        $displayBeginningBalance =~ s/inf/Infinity/;
        print
          "<strong>Beginning Balance:</strong> $displayBeginningBalance<br>\n";

        # Obtain the sum of all credits over the time period
        my $totalCredits = 0;
        {
            my $request = new MAM::Request(
                object => "Transaction",
                action => "Query",
                %tokenArgs
            );
            setFunds($request, \%funds);
            $request->addCondition("Delta",        0,          "GT");
            $request->addCondition("CreationTime", $startTime, "GE");
            $request->addCondition("CreationTime", $endTime,   "LT");
            $request->addSelection("Delta", "Sum");
            my $response = $request->getResponse();

            if ($response->getStatus() eq "Failure")
            {
                my $code    = $response->getCode();
                my $message = $response->getMessage();
                print start_html(
                    -title  => "$PRODUCT Error",
                    -script => {-code => $java_script},
                    -onLoad =>
                      "updateStatus(\"message=Aborting Fund Statement: $message.\")"
                );
                return;
            }
            my $doc  = XML::LibXML::Document->new();
            my $data = $response->getDataElement();
            $doc->setDocumentElement($data);
            foreach my $row ($data->childNodes())
            {
                my $sum =
                  ($row->getChildrenByTagName("Delta"))[0]->textContent();
                $totalCredits += $sum if $sum;
            }
        }

        # Print the total credits
        my $displayTotalCredits =
          sprintf("%20.${currency_precision}f", $totalCredits / $time_division);
        $displayTotalCredits =~ s/inf/Infinity/;
        print "<strong>Total Credits:</strong>     $displayTotalCredits<br>\n";

        # Obtain the sum of all debits over the time period
        my $totalDebits = 0;
        {
            my $request = new MAM::Request(
                object => "Transaction",
                action => "Query",
                %tokenArgs
            );
            setFunds($request, \%funds);
            $request->addCondition("Delta",        0,          "LE");
            $request->addCondition("CreationTime", $startTime, "GE");
            $request->addCondition("CreationTime", $endTime,   "LT");
            $request->addSelection("Delta", "Sum");
            my $response = $request->getResponse();

            if ($response->getStatus() eq "Failure")
            {
                my $code    = $response->getCode();
                my $message = $response->getMessage();
                print start_html(
                    -title  => "$PRODUCT Error",
                    -script => {-code => $java_script},
                    -onLoad =>
                      "updateStatus(\"message=Aborting Fund Statement: $message.\")"
                );
                return;
            }
            my $doc  = XML::LibXML::Document->new();
            my $data = $response->getDataElement();
            $doc->setDocumentElement($data);
            foreach my $row ($data->childNodes())
            {
                my $sum =
                  ($row->getChildrenByTagName("Delta"))[0]->textContent();
                $totalDebits += $sum if $sum;
            }
        }

        # Print the total debits
        my $displayTotalDebits =
          sprintf("%20.${currency_precision}f", $totalDebits / $time_division);
        $displayTotalDebits =~ s/inf/Infinity/;
        print "<strong>Total Debits:</strong>      $displayTotalDebits<br>\n";

        # Add up the ending active allocations
        my $endingBalance = 0;
        {
            my $request = new MAM::Request(
                object => "Allocation",
                action => "Query",
                %tokenArgs
            );
            setFunds($request, \%funds);
            $request->addCondition("Active", "True");
            $request->setOption("Time", $endTime);
            $request->addSelection("Amount", "Sum");
            my $response = $request->getResponse();

            if ($response->getStatus() eq "Failure")
            {
                my $code    = $response->getCode();
                my $message = $response->getMessage();
                print start_html(
                    -title  => "$PRODUCT Error",
                    -script => {-code => $java_script},
                    -onLoad =>
                      "updateStatus(\"message=Aborting Fund Statement: $message.\")"
                );
                return;
            }
            my $doc  = XML::LibXML::Document->new();
            my $data = $response->getDataElement();
            $doc->setDocumentElement($data);
            foreach my $row ($data->childNodes())
            {
                my $sum =
                  ($row->getChildrenByTagName("Amount"))[0]->textContent();
                $endingBalance += $sum if $sum;
            }
        }

        # Print the ending balance
        my $displayEndingBalance = sprintf("%20.${currency_precision}f",
            $endingBalance / $time_division);
        $displayEndingBalance =~ s/inf/Infinity/;
        print "<strong>Ending Balance:</strong>    $displayEndingBalance<br>\n";

        # Check for transactional consistency with journal state
        my $discrepancy =
          ($totalCredits + $totalDebits) - ($endingBalance - $beginningBalance);
        # Do not warn if discrepancy is nan (detectable by spaceship operator)
        if ($discrepancy && defined($discrepancy <=> $discrepancy))
        {
            printf
              "<br><strong>Warning: A discrepancy of %20d credits was detected\nbetween the logged transactions and the historical fund balances.</strong><br>\n",
              $discrepancy;
        }

        print "</div>\n";

        # Print credit and debit detail (or summary)
        {
            my $detail;
            if   ($itemize eq "True") { $detail = "Detail"; }
            else                      { $detail = "Summary"; }

            print <<END_OF_HTML;
<div id="available">
<strong>Credit $detail:</strong>
</div>
END_OF_HTML

            # Print out all of the credits
            {
                my $total = 0;
                my $colspan;

                my $request = new MAM::Request(
                    object => "Transaction",
                    action => "Query",
                    %tokenArgs
                );
                setFunds($request, \%funds);
                $request->addCondition("Delta",        0,          "GT");
                $request->addCondition("CreationTime", $startTime, "GE");
                $request->addCondition("CreationTime", $endTime,   "LT");
                if ($itemize eq "True")
                {
                    $request->addSelection("Object");
                    $request->addSelection("Action");
                    $request->addSelection("Instance");
                    $request->addSelection("CreationTime", "Sort", "", "Time");
                    $request->addSelection("Delta", "", "", "Amount");
                    $request->addSelection("Balance");
                    $colspan = 5;
                }
                else
                {
                    $request->addSelection("Object", "GroupBy");
                    $request->addSelection("Action", "GroupBy");
                    $request->addSelection("Id",     "Count", "", "Count");
                    $request->addSelection("Delta",  "Sum", "", "Amount");
                    $colspan = 3;
                }
                my $response = $request->getResponse();
                if ($response->getStatus() eq "Failure")
                {
                    my $code    = $response->getCode();
                    my $message = $response->getMessage();
                    print start_html(
                        -title  => "$PRODUCT Error",
                        -script => {-code => $java_script},
                        -onLoad =>
                          "updateStatus(\"message=Aborting Fund Statement: $message.\")"
                    );
                    return;
                }
                my $data = $response->getDataElement();
                my $doc  = XML::LibXML::Document->new();
                $doc->setDocumentElement($data);
                my @rows = $data->childNodes();

                if (@rows > 0)
                {
                    # Print Table Header
                    print <<END_OF_HTML;
<table id="funds" cellspacing="0">
  <tr>
END_OF_HTML
                    foreach my $col (($rows[0])->childNodes())
                    {
                        my $name = $col->nodeName();
                        print "<th><div>$name</div></th>\n";
                    }
                    print "</tr>\n";

                    # Print Table Body
                    foreach my $row (@rows)
                    {
                        print "<tr>\n";
                        foreach my $col ($row->childNodes())
                        {
                            my $name      = $col->nodeName();
                            my $value     = $col->textContent();
                            my $alignment = "";
                            if ($name eq "Amount" || $name eq "Balance")
                            {
                                $total += $value;
                                $value = sprintf("%.${currency_precision}f",
                                    $value / $time_division);
                                if ($value =~ /inf/)
                                {
                                    $value =~ s/inf/Infinity/;
                                }
                                else
                                {
                                    $alignment = " align=\"right\"";
                                }
                            }
                            $value = "&nbsp;"
                              if $value eq "";    # So border will be drawn
                            print "<td$alignment>$value</td>\n";
                        }
                        print "</tr>\n";
                    }

                    my $alignment = "";
                    $total = sprintf("%.${currency_precision}f",
                        $total / $time_division);
                    if ($total =~ /inf/)
                    {
                        $total =~ s/inf/Infinity/;
                    }
                    else
                    {
                        $alignment = " align=\"right\"";
                    }
                    # Print Table Totals
                    print <<END_OF_HTML;
<tr class="totals">
<td class="Total" colspan="$colspan">Total:</td><td$alignment class="amount">$total</td>
</tr>
</table>
END_OF_HTML
                }
            }

            print <<END_OF_HTML;
<div id="available">
<strong>Debit $detail:</strong>
</div>
END_OF_HTML

            # Print out all of the debits
            {
                my $total = 0;
                my $colspan;

                my $request = new MAM::Request(
                    object => "Transaction",
                    action => "Query",
                    %tokenArgs
                );
                setFunds($request, \%funds);
                $request->addCondition("Delta",        0,          "LE");
                $request->addCondition("CreationTime", $startTime, "GE");
                $request->addCondition("CreationTime", $endTime,   "LT");
                if ($itemize eq "True")
                {
                    $colspan = 5;
                    $request->addSelection("Object");
                    $request->addSelection("Action");
                    $request->addSelection("Instance");
                    foreach my $name (@discriminators)
                    {
                        $request->addSelection($name);
                        $colspan++;
                    }
                    $request->addSelection("CreationTime", "Sort", "", "Time");
                    $request->addSelection("Delta", "", "", "Amount");
                    $request->addSelection("Balance");
                }
                else
                {
                    $colspan = 3;
                    $request->addSelection("Object", "GroupBy");
                    $request->addSelection("Action", "GroupBy");
                    foreach my $name (@discriminators)
                    {
                        $request->addSelection($name, "GroupBy");
                        $colspan++;
                    }
                    $request->addSelection("Id",    "Count", "", "Count");
                    $request->addSelection("Delta", "Sum",   "", "Amount");
                }
                my $response = $request->getResponse();
                if ($response->getStatus() eq "Failure")
                {
                    my $code    = $response->getCode();
                    my $message = $response->getMessage();
                    print start_html(
                        -title  => "$PRODUCT Error",
                        -script => {-code => $java_script},
                        -onLoad =>
                          "updateStatus(\"message=Aborting Fund Statement: $message.\")"
                    );
                    return;
                }
                my $data = $response->getDataElement();
                my $doc  = XML::LibXML::Document->new();
                $doc->setDocumentElement($data);
                my @rows = $data->childNodes();

                if (@rows > 0)
                {
                    # Print Table Header
                    print <<END_OF_HTML;
<table id="funds" cellspacing="0">
  <tr>
END_OF_HTML
                    foreach my $col (($rows[0])->childNodes())
                    {
                        my $name = $col->nodeName();
                        print "<th><div>$name</div></th>\n";
                    }
                    print "</tr>\n";

                    # Print Table Body
                    foreach my $row (@rows)
                    {
                        print "<tr>\n";
                        foreach my $col ($row->childNodes())
                        {
                            my $name      = $col->nodeName();
                            my $value     = $col->textContent();
                            my $alignment = "";
                            if ($name eq "Amount" || $name eq "Balance")
                            {
                                $total += $value;
                                $value = sprintf("%.${currency_precision}f",
                                    $value / $time_division);
                                if ($value =~ /inf/)
                                {
                                    $value =~ s/inf/Infinity/;
                                }
                                else
                                {
                                    $alignment = " align=\"right\"";
                                }
                            }
                            $value = "&nbsp;"
                              if $value eq "";    # So border will be drawn
                            print "<td$alignment>$value</td>\n";
                        }
                        print "</tr>\n";
                    }

                    my $alignment = "";
                    $total = sprintf("%.${currency_precision}f",
                        $total / $time_division);
                    if ($total =~ /inf/)
                    {
                        $total =~ s/inf/Infinity/;
                    }
                    else
                    {
                        $alignment = " align=\"right\"";
                    }
                    # Print Table Totals
                    print <<END_OF_HTML;
<tr class="totals">
<td class="Total" colspan="$colspan">Total:</td><td$alignment class="amount">$total</td>
</tr>
</table>
END_OF_HTML
                }
            }
        }
    }
}

# Subroutine that builds the conditions list for multiple funds
sub setFunds
{
    my ($request, $funds_hashref) = @_;

    my @funds = keys %{$funds_hashref};
    my $count    = 0;
    my $total    = scalar @funds;

    foreach my $fund (@funds)
    {
        my ($conj, $group);

        $count++;
        if ($count == 1)
        {
            $conj = "And";
            if   ($count == $total) { $group = "0"; }
            else                    { $group = "+1"; }
        }
        else
        {
            $conj = "Or";
            if   ($count == $total) { $group = "-1"; }
            else                    { $group = "0"; }
        }
        $request->addCondition("Fund", $fund, "EQ", $conj, $group);
    }
}

